# encoding: utf-8

"""Section-related custom element classes"""

from __future__ import absolute_import, division, print_function, unicode_literals

from copy import deepcopy

from docx.enum.section import WD_HEADER_FOOTER, WD_ORIENTATION, WD_SECTION_START
from docx.oxml.simpletypes import ST_SignedTwipsMeasure, ST_TwipsMeasure, XsdString
from docx.oxml.xmlchemy import (
    BaseOxmlElement,
    OptionalAttribute,
    RequiredAttribute,
    ZeroOrMore,
    ZeroOrOne,
)


class CT_HdrFtr(BaseOxmlElement):
    """`w:hdr` and `w:ftr`, the root element for header and footer part respectively"""

    p = ZeroOrMore('w:p', successors=())
    tbl = ZeroOrMore('w:tbl', successors=())


class CT_HdrFtrRef(BaseOxmlElement):
    """`w:headerReference` and `w:footerReference` elements"""

    type_ = RequiredAttribute('w:type', WD_HEADER_FOOTER)
    rId = RequiredAttribute('r:id', XsdString)


class CT_PageMar(BaseOxmlElement):
    """
    ``<w:pgMar>`` element, defining page margins.
    """
    top = OptionalAttribute('w:top', ST_SignedTwipsMeasure)
    right = OptionalAttribute('w:right', ST_TwipsMeasure)
    bottom = OptionalAttribute('w:bottom', ST_SignedTwipsMeasure)
    left = OptionalAttribute('w:left', ST_TwipsMeasure)
    header = OptionalAttribute('w:header', ST_TwipsMeasure)
    footer = OptionalAttribute('w:footer', ST_TwipsMeasure)
    gutter = OptionalAttribute('w:gutter', ST_TwipsMeasure)


class CT_PageSz(BaseOxmlElement):
    """
    ``<w:pgSz>`` element, defining page dimensions and orientation.
    """
    w = OptionalAttribute('w:w', ST_TwipsMeasure)
    h = OptionalAttribute('w:h', ST_TwipsMeasure)
    orient = OptionalAttribute(
        'w:orient', WD_ORIENTATION, default=WD_ORIENTATION.PORTRAIT
    )

class CT_SectPr(BaseOxmlElement):
    """`w:sectPr` element, the container element for section properties"""

    _tag_seq = (
        'w:footnotePr', 'w:endnotePr', 'w:type', 'w:pgSz', 'w:pgMar', 'w:paperSrc',
        'w:pgBorders', 'w:lnNumType', 'w:pgNumType', 'w:cols', 'w:formProt', 'w:vAlign',
        'w:noEndnote', 'w:titlePg', 'w:textDirection', 'w:bidi', 'w:rtlGutter',
        'w:docGrid', 'w:printerSettings', 'w:sectPrChange',
    )
    headerReference = ZeroOrMore("w:headerReference", successors=_tag_seq)
    footerReference = ZeroOrMore("w:footerReference", successors=_tag_seq)
    type = ZeroOrOne("w:type", successors=_tag_seq[3:])
    pgSz = ZeroOrOne("w:pgSz", successors=_tag_seq[4:])
    pgMar = ZeroOrOne("w:pgMar", successors=_tag_seq[5:])
    titlePg = ZeroOrOne("w:titlePg", successors=_tag_seq[14:])
    del _tag_seq

    def add_footerReference(self, type_, rId):
        """Return newly added CT_HdrFtrRef element of *type_* with *rId*.

        The element tag is `w:footerReference`.
        """
        footerReference = self._add_footerReference()
        footerReference.type_ = type_
        footerReference.rId = rId
        return footerReference

    def add_headerReference(self, type_, rId):
        """Return newly added CT_HdrFtrRef element of *type_* with *rId*.

        The element tag is `w:headerReference`.
        """
        headerReference = self._add_headerReference()
        headerReference.type_ = type_
        headerReference.rId = rId
        return headerReference


    @property
    def bottom_margin(self):
        """
        The value of the ``w:bottom`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.bottom

    @bottom_margin.setter
    def bottom_margin(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.bottom = value

    def clone(self):
        """
        Return an exact duplicate of this ``<w:sectPr>`` element tree
        suitable for use in adding a section break. All rsid* attributes are
        removed from the root ``<w:sectPr>`` element.
        """
        clone_sectPr = deepcopy(self)
        clone_sectPr.attrib.clear()
        return clone_sectPr

    @property
    def footer(self):
        """
        The value of the ``w:footer`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.footer

    @footer.setter
    def footer(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.footer = value

    def get_footerReference(self, type_):
        """Return footerReference element of *type_* or None if not present."""
        path = "./w:footerReference[@w:type='%s']" % WD_HEADER_FOOTER.to_xml(type_)
        footerReferences = self.xpath(path)
        if not footerReferences:
            return None
        return footerReferences[0]

    def get_headerReference(self, type_):
        """Return headerReference element of *type_* or None if not present."""
        matching_headerReferences = self.xpath(
            "./w:headerReference[@w:type='%s']" % WD_HEADER_FOOTER.to_xml(type_)
        )
        if len(matching_headerReferences) == 0:
            return None
        return matching_headerReferences[0]

    @property
    def gutter(self):
        """
        The value of the ``w:gutter`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.gutter

    @gutter.setter
    def gutter(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.gutter = value

    @property
    def header(self):
        """
        The value of the ``w:header`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.header

    @header.setter
    def header(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.header = value

    @property
    def left_margin(self):
        """
        The value of the ``w:left`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.left

    @left_margin.setter
    def left_margin(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.left = value

    @property
    def orientation(self):
        """
        The member of the ``WD_ORIENTATION`` enumeration corresponding to the
        value of the ``orient`` attribute of the ``<w:pgSz>`` child element,
        or ``WD_ORIENTATION.PORTRAIT`` if not present.
        """
        pgSz = self.pgSz
        if pgSz is None:
            return WD_ORIENTATION.PORTRAIT
        return pgSz.orient

    @orientation.setter
    def orientation(self, value):
        pgSz = self.get_or_add_pgSz()
        pgSz.orient = value

    @property
    def page_height(self):
        """
        Value in EMU of the ``h`` attribute of the ``<w:pgSz>`` child
        element, or |None| if not present.
        """
        pgSz = self.pgSz
        if pgSz is None:
            return None
        return pgSz.h

    @page_height.setter
    def page_height(self, value):
        pgSz = self.get_or_add_pgSz()
        pgSz.h = value

    @property
    def page_width(self):
        """
        Value in EMU of the ``w`` attribute of the ``<w:pgSz>`` child
        element, or |None| if not present.
        """
        pgSz = self.pgSz
        if pgSz is None:
            return None
        return pgSz.w

    @page_width.setter
    def page_width(self, value):
        pgSz = self.get_or_add_pgSz()
        pgSz.w = value

    @property
    def preceding_sectPr(self):
        """sectPr immediately preceding this one or None if this is the first."""
        # ---[1] predicate returns list of zero or one value---
        preceding_sectPrs = self.xpath("./preceding::w:sectPr[1]")
        return preceding_sectPrs[0] if len(preceding_sectPrs) > 0 else None

    def remove_footerReference(self, type_):
        """Return rId of w:footerReference child of *type_* after removing it."""
        footerReference = self.get_footerReference(type_)
        rId = footerReference.rId
        self.remove(footerReference)
        return rId

    def remove_headerReference(self, type_):
        """Return rId of w:headerReference child of *type_* after removing it."""
        headerReference = self.get_headerReference(type_)
        rId = headerReference.rId
        self.remove(headerReference)
        return rId

    @property
    def right_margin(self):
        """
        The value of the ``w:right`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.right

    @right_margin.setter
    def right_margin(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.right = value

    @property
    def start_type(self):
        """
        The member of the ``WD_SECTION_START`` enumeration corresponding to
        the value of the ``val`` attribute of the ``<w:type>`` child element,
        or ``WD_SECTION_START.NEW_PAGE`` if not present.
        """
        type = self.type
        if type is None or type.val is None:
            return WD_SECTION_START.NEW_PAGE
        return type.val

    @start_type.setter
    def start_type(self, value):
        if value is None or value is WD_SECTION_START.NEW_PAGE:
            self._remove_type()
            return
        type = self.get_or_add_type()
        type.val = value

    @property
    def titlePg_val(self):
        """Value of `w:titlePg/@val` or |None| if not present"""
        titlePg = self.titlePg
        if titlePg is None:
            return False
        return titlePg.val

    @titlePg_val.setter
    def titlePg_val(self, value):
        if value in [None, False]:
            self._remove_titlePg()
        else:
            self.get_or_add_titlePg().val = value

    @property
    def top_margin(self):
        """
        The value of the ``w:top`` attribute in the ``<w:pgMar>`` child
        element, as a |Length| object, or |None| if either the element or the
        attribute is not present.
        """
        pgMar = self.pgMar
        if pgMar is None:
            return None
        return pgMar.top

    @top_margin.setter
    def top_margin(self, value):
        pgMar = self.get_or_add_pgMar()
        pgMar.top = value


class CT_SectType(BaseOxmlElement):
    """
    ``<w:sectType>`` element, defining the section start type.
    """
    val = OptionalAttribute('w:val', WD_SECTION_START)



# encoding: utf-8

"""
Unit test suite for the docx.opc.coreprops module
"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import pytest

from datetime import datetime

from docx.opc.coreprops import CoreProperties
from docx.oxml import parse_xml


class DescribeCoreProperties(object):

    def it_knows_the_string_property_values(self, text_prop_get_fixture):
        core_properties, prop_name, expected_value = text_prop_get_fixture
        actual_value = getattr(core_properties, prop_name)
        assert actual_value == expected_value

    def it_can_change_the_string_property_values(self, text_prop_set_fixture):
        core_properties, prop_name, value, expected_xml = text_prop_set_fixture
        setattr(core_properties, prop_name, value)
        assert core_properties._element.xml == expected_xml

    def it_knows_the_date_property_values(self, date_prop_get_fixture):
        core_properties, prop_name, expected_datetime = date_prop_get_fixture
        actual_datetime = getattr(core_properties, prop_name)
        assert actual_datetime == expected_datetime

    def it_can_change_the_date_property_values(self, date_prop_set_fixture):
        core_properties, prop_name, value, expected_xml = (
            date_prop_set_fixture
        )
        setattr(core_properties, prop_name, value)
        assert core_properties._element.xml == expected_xml

    def it_knows_the_revision_number(self, revision_get_fixture):
        core_properties, expected_revision = revision_get_fixture
        assert core_properties.revision == expected_revision

    def it_can_change_the_revision_number(self, revision_set_fixture):
        core_properties, revision, expected_xml = revision_set_fixture
        core_properties.revision = revision
        assert core_properties._element.xml == expected_xml

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        ('created',      datetime(2012, 11, 17, 16, 37, 40)),
        ('last_printed', datetime(2014,  6,  4,  4, 28)),
        ('modified',     None),
    ])
    def date_prop_get_fixture(self, request, core_properties):
        prop_name, expected_datetime = request.param
        return core_properties, prop_name, expected_datetime

    @pytest.fixture(params=[
        ('created', 'dcterms:created', datetime(2001, 2, 3, 4, 5),
         '2001-02-03T04:05:00Z', ' xsi:type="dcterms:W3CDTF"'),
        ('last_printed', 'cp:lastPrinted', datetime(2014, 6, 4, 4),
         '2014-06-04T04:00:00Z', ''),
        ('modified', 'dcterms:modified', datetime(2005, 4, 3, 2, 1),
         '2005-04-03T02:01:00Z', ' xsi:type="dcterms:W3CDTF"'),
    ])
    def date_prop_set_fixture(self, request):
        prop_name, tagname, value, str_val, attrs = request.param
        coreProperties = self.coreProperties(None, None)
        core_properties = CoreProperties(parse_xml(coreProperties))
        expected_xml = self.coreProperties(tagname, str_val, attrs)
        return core_properties, prop_name, value, expected_xml

    @pytest.fixture(params=[
        ('42', 42), (None, 0), ('foobar', 0), ('-17', 0), ('32.7', 0)
    ])
    def revision_get_fixture(self, request):
        str_val, expected_revision = request.param
        tagname = '' if str_val is None else 'cp:revision'
        coreProperties = self.coreProperties(tagname, str_val)
        core_properties = CoreProperties(parse_xml(coreProperties))
        return core_properties, expected_revision

    @pytest.fixture(params=[
        (42, '42'),
    ])
    def revision_set_fixture(self, request):
        value, str_val = request.param
        coreProperties = self.coreProperties(None, None)
        core_properties = CoreProperties(parse_xml(coreProperties))
        expected_xml = self.coreProperties('cp:revision', str_val)
        return core_properties, value, expected_xml

    @pytest.fixture(params=[
        ('author',           'python-docx'),
        ('category',         ''),
        ('comments',         ''),
        ('content_status',   'DRAFT'),
        ('identifier',       'GXS 10.2.1ab'),
        ('keywords',         'foo bar baz'),
        ('language',         'US-EN'),
        ('last_modified_by', 'Steve Canny'),
        ('subject',          'Spam'),
        ('title',            'Word Document'),
        ('version',          '1.2.88'),
    ])
    def text_prop_get_fixture(self, request, core_properties):
        prop_name, expected_value = request.param
        return core_properties, prop_name, expected_value

    @pytest.fixture(params=[
        ('author',           'dc:creator',        'scanny'),
        ('category',         'cp:category',       'silly stories'),
        ('comments',         'dc:description',    'Bar foo to you'),
        ('content_status',   'cp:contentStatus',  'FINAL'),
        ('identifier',       'dc:identifier',     'GT 5.2.xab'),
        ('keywords',         'cp:keywords',       'dog cat moo'),
        ('language',         'dc:language',       'GB-EN'),
        ('last_modified_by', 'cp:lastModifiedBy', 'Billy Bob'),
        ('subject',          'dc:subject',        'Eggs'),
        ('title',            'dc:title',          'Dissertation'),
        ('version',          'cp:version',        '81.2.8'),
    ])
    def text_prop_set_fixture(self, request):
        prop_name, tagname, value = request.param
        coreProperties = self.coreProperties(None, None)
        core_properties = CoreProperties(parse_xml(coreProperties))
        expected_xml = self.coreProperties(tagname, value)
        return core_properties, prop_name, value, expected_xml

    # fixture components ---------------------------------------------

    def coreProperties(self, tagname, str_val, attrs=''):
        tmpl = (
            '<cp:coreProperties xmlns:cp="http://schemas.openxmlformats.org/'
            'package/2006/metadata/core-properties" xmlns:dc="http://purl.or'
            'g/dc/elements/1.1/" xmlns:dcmitype="http://purl.org/dc/dcmitype'
            '/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:xsi="http://'
            'www.w3.org/2001/XMLSchema-instance">%s</cp:coreProperties>\n'
        )
        if not tagname:
            child_element = ''
        elif not str_val:
            child_element = '\n  <%s%s/>\n' % (tagname, attrs)
        else:
            child_element = (
                '\n  <%s%s>%s</%s>\n' % (tagname, attrs, str_val, tagname)
            )
        return tmpl % child_element

    @pytest.fixture
    def core_properties(self):
        element = parse_xml(
            b'<?xml version=\'1.0\' encoding=\'UTF-8\' standalone=\'yes\'?>'
            b'\n<cp:coreProperties xmlns:cp="http://schemas.openxmlformats.o'
            b'rg/package/2006/metadata/core-properties" xmlns:dc="http://pur'
            b'l.org/dc/elements/1.1/" xmlns:dcmitype="http://purl.org/dc/dcm'
            b'itype/" xmlns:dcterms="http://purl.org/dc/terms/" xmlns:xsi="h'
            b'ttp://www.w3.org/2001/XMLSchema-instance">\n'
            b'  <cp:contentStatus>DRAFT</cp:contentStatus>\n'
            b'  <dc:creator>python-docx</dc:creator>\n'
            b'  <dcterms:created xsi:type="dcterms:W3CDTF">2012-11-17T11:07:'
            b'40-05:30</dcterms:created>\n'
            b'  <dc:description/>\n'
            b'  <dc:identifier>GXS 10.2.1ab</dc:identifier>\n'
            b'  <dc:language>US-EN</dc:language>\n'
            b'  <cp:lastPrinted>2014-06-04T04:28:00Z</cp:lastPrinted>\n'
            b'  <cp:keywords>foo bar baz</cp:keywords>\n'
            b'  <cp:lastModifiedBy>Steve Canny</cp:lastModifiedBy>\n'
            b'  <cp:revision>4</cp:revision>\n'
            b'  <dc:subject>Spam</dc:subject>\n'
            b'  <dc:title>Word Document</dc:title>\n'
            b'  <cp:version>1.2.88</cp:version>\n'
            b'</cp:coreProperties>\n'
        )
        return CoreProperties(element)



# encoding: utf-8

"""
Shared code for unit test data builders
"""

from __future__ import absolute_import, print_function, unicode_literals

from docx.oxml import parse_xml
from docx.oxml.ns import nsdecls


class BaseBuilder(object):
    """
    Provides common behavior for all data builders.
    """
    def __init__(self):
        self._empty = False
        self._nsdecls = ''
        self._text = ''
        self._xmlattrs = []
        self._xmlattr_method_map = {}
        for attr_name in self.__attrs__:
            base_name = (
                attr_name.split(':')[1] if ':' in attr_name else attr_name
            )
            method_name = 'with_%s' % base_name
            self._xmlattr_method_map[method_name] = attr_name
        self._child_bldrs = []

    def __getattr__(self, name):
        """
        Intercept attribute access to generalize "with_{xmlattr_name}()"
        methods.
        """
        if name in self._xmlattr_method_map:
            def with_xmlattr(value):
                xmlattr_name = self._xmlattr_method_map[name]
                self._set_xmlattr(xmlattr_name, value)
                return self
            return with_xmlattr
        else:
            tmpl = "'%s' object has no attribute '%s'"
            raise AttributeError(tmpl % (self.__class__.__name__, name))

    def clear(self):
        """
        Reset this builder back to initial state so it can be reused within
        a single test.
        """
        BaseBuilder.__init__(self)
        return self

    @property
    def element(self):
        """
        Element parsed from XML generated by builder in current state
        """
        elm = parse_xml(self.xml())
        return elm

    def with_child(self, child_bldr):
        """
        Cause new child element specified by *child_bldr* to be appended to
        the children of this element.
        """
        self._child_bldrs.append(child_bldr)
        return self

    def with_text(self, text):
        """
        Cause *text* to be placed between the start and end tags of this
        element. Not robust enough for mixed elements, intended only for
        elements having no child elements.
        """
        self._text = text
        return self

    def with_nsdecls(self, *nspfxs):
        """
        Cause the element to contain namespace declarations. By default, the
        namespace prefixes defined in the Builder class are used. These can
        be overridden by providing exlicit prefixes, e.g.
        ``with_nsdecls('a', 'r')``.
        """
        if not nspfxs:
            nspfxs = self.__nspfxs__
        self._nsdecls = ' %s' % nsdecls(*nspfxs)
        return self

    def xml(self, indent=0):
        """
        Return element XML based on attribute settings
        """
        indent_str = ' ' * indent
        if self._is_empty:
            xml = '%s%s\n' % (indent_str, self._empty_element_tag)
        else:
            xml = '%s\n' % self._non_empty_element_xml(indent)
        return xml

    def xml_bytes(self, indent=0):
        return self.xml(indent=indent).encode('utf-8')

    @property
    def _empty_element_tag(self):
        return '<%s%s%s/>' % (self.__tag__, self._nsdecls, self._xmlattrs_str)

    @property
    def _end_tag(self):
        return '</%s>' % self.__tag__

    @property
    def _is_empty(self):
        return len(self._child_bldrs) == 0 and len(self._text) == 0

    def _non_empty_element_xml(self, indent):
        indent_str = ' ' * indent
        if self._text:
            xml = ('%s%s%s%s' %
                   (indent_str, self._start_tag, self._text, self._end_tag))
        else:
            xml = '%s%s\n' % (indent_str, self._start_tag)
            for child_bldr in self._child_bldrs:
                xml += child_bldr.xml(indent+2)
            xml += '%s%s' % (indent_str, self._end_tag)
        return xml

    def _set_xmlattr(self, xmlattr_name, value):
        xmlattr_str = ' %s="%s"' % (xmlattr_name, str(value))
        self._xmlattrs.append(xmlattr_str)

    @property
    def _start_tag(self):
        return '<%s%s%s>' % (self.__tag__, self._nsdecls, self._xmlattrs_str)

    @property
    def _xmlattrs_str(self):
        """
        Return all element attributes as a string, like ' foo="bar" x="1"'.
        """
        return ''.join(self._xmlattrs)



# encoding: utf-8

from __future__ import absolute_import, division, print_function

from .constants import MIME_TYPE
from .exceptions import InvalidImageStreamError
from .helpers import BIG_ENDIAN, StreamReader
from .image import BaseImageHeader
import struct

class Emf(BaseImageHeader):
    """
    Image header parser for PNG images
    """
    @property
    def content_type(self):
        """
        MIME content type for this image, unconditionally `image/png` for
        PNG images.
        """
        return MIME_TYPE.EMF

    @property
    def default_ext(self):
        """
        Default filename extension, always 'png' for PNG images.
        """
        return 'emf'

    @classmethod
    def from_stream(cls, stream,filename=None):
        """
        Return a |Emf| instance having header properties parsed from image in
        *stream*.
        """

        """
        @0 DWORD iType; // fixed
        @4 DWORD nSize; // var 
        @8 RECTL rclBounds;
        @24 RECTL rclFrame; // .01 millimeter units L T R B
        @40 DWORD dSignature; // ENHMETA_SIGNATURE = 0x464D4520
        DWORD nVersion;
        DWORD nBytes;
        DWORD nRecords;
        WORD  nHandles;
        WORD  sReserved;
        DWORD nDescription;
        DWORD offDescription;
        DWORD nPalEntries;
        SIZEL szlDevice;
        SIZEL szlMillimeters;
        """
        stream.seek(0)
        x = stream.read(40)
        stream.seek(0)
        iType,nSize = struct.unpack("ii",x[0:8])
        rclBounds = struct.unpack("iiii",x[8:24])
        rclFrame = struct.unpack("iiii",x[24:40])

        dpi = 300
        horz_dpi = dpi
        vert_dpi = dpi
        mmwidth = (rclFrame[2]-rclFrame[0])/100.0
        mmheight = (rclFrame[3]-rclFrame[1])/100.0
        px_width = int(mmwidth*dpi*0.03937008)
        px_height = int(mmheight*dpi*0.03937008)

        #1 dot/inch  =  0.03937008 pixel/millimeter
        return cls(px_width,px_height,horz_dpi,vert_dpi)


# encoding: utf-8

"""
Test data builders for numbering part XML elements
"""

from ...unitdata import BaseBuilder


class CT_NumBuilder(BaseBuilder):
    __tag__ = 'w:num'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:numId')


class CT_NumberingBuilder(BaseBuilder):
    __tag__ = 'w:numbering'
    __nspfxs__ = ('w',)
    __attrs__ = ()


def a_num():
    return CT_NumBuilder()


def a_numbering():
    return CT_NumberingBuilder()



# encoding: utf-8

"""Unit test suite for docx.image.jpeg module"""

from __future__ import absolute_import, print_function

import pytest

from docx.compat import BytesIO
from docx.image.constants import JPEG_MARKER_CODE, MIME_TYPE
from docx.image.helpers import BIG_ENDIAN, StreamReader
from docx.image.jpeg import (
    _App0Marker,
    _App1Marker,
    Exif,
    Jfif,
    _JfifMarkers,
    Jpeg,
    _Marker,
    _MarkerFactory,
    _MarkerFinder,
    _MarkerParser,
    _SofMarker,
)
from docx.image.tiff import Tiff

from ..unitutil.mock import (
    ANY,
    call,
    class_mock,
    initializer_mock,
    instance_mock,
    method_mock,
)


class DescribeJpeg(object):

    def it_knows_its_content_type(self):
        jpeg = Jpeg(None, None, None, None)
        assert jpeg.content_type == MIME_TYPE.JPEG

    def it_knows_its_default_ext(self):
        jpeg = Jpeg(None, None, None, None)
        assert jpeg.default_ext == 'jpg'

    class DescribeExif(object):

        def it_can_construct_from_an_exif_stream(self, from_exif_fixture):
            # fixture ----------------------
            stream_, _JfifMarkers_, cx, cy, horz_dpi, vert_dpi = (
                from_exif_fixture
            )
            # exercise ---------------------
            exif = Exif.from_stream(stream_)
            # verify -----------------------
            _JfifMarkers_.from_stream.assert_called_once_with(stream_)
            assert isinstance(exif, Exif)
            assert exif.px_width == cx
            assert exif.px_height == cy
            assert exif.horz_dpi == horz_dpi
            assert exif.vert_dpi == vert_dpi

    class DescribeJfif(object):

        def it_can_construct_from_a_jfif_stream(self, from_jfif_fixture):
            stream_, _JfifMarkers_, cx, cy, horz_dpi, vert_dpi = (
                from_jfif_fixture
            )
            jfif = Jfif.from_stream(stream_)
            _JfifMarkers_.from_stream.assert_called_once_with(stream_)
            assert isinstance(jfif, Jfif)
            assert jfif.px_width == cx
            assert jfif.px_height == cy
            assert jfif.horz_dpi == horz_dpi
            assert jfif.vert_dpi == vert_dpi

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def from_exif_fixture(self, stream_, _JfifMarkers_, jfif_markers_):
        px_width, px_height = 111, 222
        horz_dpi, vert_dpi = 333, 444
        jfif_markers_.sof.px_width = px_width
        jfif_markers_.sof.px_height = px_height
        jfif_markers_.app1.horz_dpi = horz_dpi
        jfif_markers_.app1.vert_dpi = vert_dpi
        return (
            stream_, _JfifMarkers_, px_width, px_height, horz_dpi, vert_dpi
        )

    @pytest.fixture
    def from_jfif_fixture(self, stream_, _JfifMarkers_, jfif_markers_):
        px_width, px_height = 111, 222
        horz_dpi, vert_dpi = 333, 444
        jfif_markers_.sof.px_width = px_width
        jfif_markers_.sof.px_height = px_height
        jfif_markers_.app0.horz_dpi = horz_dpi
        jfif_markers_.app0.vert_dpi = vert_dpi
        return (
            stream_, _JfifMarkers_, px_width, px_height, horz_dpi, vert_dpi
        )

    @pytest.fixture
    def _JfifMarkers_(self, request, jfif_markers_):
        _JfifMarkers_ = class_mock(request, 'docx.image.jpeg._JfifMarkers')
        _JfifMarkers_.from_stream.return_value = jfif_markers_
        return _JfifMarkers_

    @pytest.fixture
    def jfif_markers_(self, request):
        return instance_mock(request, _JfifMarkers)

    @pytest.fixture
    def stream_(self, request):
        return instance_mock(request, BytesIO)


class Describe_JfifMarkers(object):

    def it_can_construct_from_a_jfif_stream(
            self, stream_, _MarkerParser_, _JfifMarkers__init_, soi_, app0_, sof_, sos_
    ):
        marker_lst = [soi_, app0_, sof_, sos_]

        jfif_markers = _JfifMarkers.from_stream(stream_)

        _MarkerParser_.from_stream.assert_called_once_with(stream_)
        _JfifMarkers__init_.assert_called_once_with(ANY, marker_lst)
        assert isinstance(jfif_markers, _JfifMarkers)

    def it_can_find_the_APP0_marker(self, app0_fixture):
        jfif_markers, app0_ = app0_fixture
        app0 = jfif_markers.app0
        assert app0 is app0_

    def it_can_find_the_APP1_marker(self, app1_fixture):
        jfif_markers, app1_ = app1_fixture
        app1 = jfif_markers.app1
        assert app1 is app1_

    def it_raises_if_it_cant_find_the_APP0_marker(self, no_app0_fixture):
        jfif_markers = no_app0_fixture
        with pytest.raises(KeyError):
            jfif_markers.app0

    def it_raises_if_it_cant_find_the_APP1_marker(self, no_app1_fixture):
        jfif_markers = no_app1_fixture
        with pytest.raises(KeyError):
            jfif_markers.app1

    def it_can_find_the_SOF_marker(self, sof_fixture):
        jfif_markers, sof_ = sof_fixture
        sof = jfif_markers.sof
        assert sof is sof_

    def it_raises_if_it_cant_find_the_SOF_marker(self, no_sof_fixture):
        jfif_markers = no_sof_fixture
        with pytest.raises(KeyError):
            jfif_markers.sof

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def app0_(self, request):
        return instance_mock(
            request, _App0Marker, marker_code=JPEG_MARKER_CODE.APP0
        )

    @pytest.fixture
    def app1_(self, request):
        return instance_mock(
            request, _App1Marker, marker_code=JPEG_MARKER_CODE.APP1
        )

    @pytest.fixture
    def app0_fixture(self, soi_, app0_, eoi_):
        markers = (soi_, app0_, eoi_)
        jfif_markers = _JfifMarkers(markers)
        return jfif_markers, app0_

    @pytest.fixture
    def app1_fixture(self, soi_, app1_, eoi_):
        markers = (soi_, app1_, eoi_)
        jfif_markers = _JfifMarkers(markers)
        return jfif_markers, app1_

    @pytest.fixture
    def eoi_(self, request):
        return instance_mock(
            request, _SofMarker, marker_code=JPEG_MARKER_CODE.EOI
        )

    @pytest.fixture
    def _JfifMarkers__init_(self, request):
        return initializer_mock(request, _JfifMarkers)

    @pytest.fixture
    def marker_parser_(self, request, markers_all_):
        marker_parser_ = instance_mock(request, _MarkerParser)
        marker_parser_.iter_markers.return_value = markers_all_
        return marker_parser_

    @pytest.fixture
    def _MarkerParser_(self, request, marker_parser_):
        _MarkerParser_ = class_mock(request, 'docx.image.jpeg._MarkerParser')
        _MarkerParser_.from_stream.return_value = marker_parser_
        return _MarkerParser_

    @pytest.fixture
    def markers_all_(self, request, soi_, app0_, sof_, sos_, eoi_):
        return [soi_, app0_, sof_, sos_, eoi_]

    @pytest.fixture
    def no_app0_fixture(self, soi_, eoi_):
        markers = (soi_, eoi_)
        return _JfifMarkers(markers)

    @pytest.fixture
    def no_app1_fixture(self, soi_, eoi_):
        markers = (soi_, eoi_)
        return _JfifMarkers(markers)

    @pytest.fixture
    def no_sof_fixture(self, soi_, eoi_):
        markers = (soi_, eoi_)
        return _JfifMarkers(markers)

    @pytest.fixture
    def sof_(self, request):
        return instance_mock(
            request, _SofMarker, marker_code=JPEG_MARKER_CODE.SOF0
        )

    @pytest.fixture
    def sof_fixture(self, soi_, sof_, eoi_):
        markers = (soi_, sof_, eoi_)
        jfif_markers = _JfifMarkers(markers)
        return jfif_markers, sof_

    @pytest.fixture
    def soi_(self, request):
        return instance_mock(
            request, _Marker, marker_code=JPEG_MARKER_CODE.SOI
        )

    @pytest.fixture
    def sos_(self, request):
        return instance_mock(
            request, _Marker, marker_code=JPEG_MARKER_CODE.SOS
        )

    @pytest.fixture
    def stream_(self, request):
        return instance_mock(request, BytesIO)


class Describe_Marker(object):

    def it_can_construct_from_a_stream_and_offset(self, from_stream_fixture):
        stream, marker_code, offset, _Marker__init_, length = from_stream_fixture

        marker = _Marker.from_stream(stream, marker_code, offset)

        _Marker__init_.assert_called_once_with(ANY, marker_code, offset, length)
        assert isinstance(marker, _Marker)

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        (JPEG_MARKER_CODE.SOI,  2,  0),
        (JPEG_MARKER_CODE.APP0, 4, 16),
    ])
    def from_stream_fixture(self, request, _Marker__init_):
        marker_code, offset, length = request.param
        bytes_ = b'\xFF\xD8\xFF\xE0\x00\x10'
        stream_reader = StreamReader(BytesIO(bytes_), BIG_ENDIAN)
        return stream_reader, marker_code, offset, _Marker__init_, length

    @pytest.fixture
    def _Marker__init_(self, request):
        return initializer_mock(request, _Marker)


class Describe_App0Marker(object):

    def it_can_construct_from_a_stream_and_offset(self, _App0Marker__init_):
        bytes_ = b'\x00\x10JFIF\x00\x01\x01\x01\x00\x2A\x00\x18'
        marker_code, offset, length = JPEG_MARKER_CODE.APP0, 0, 16
        density_units, x_density, y_density = 1, 42, 24
        stream = StreamReader(BytesIO(bytes_), BIG_ENDIAN)

        app0_marker = _App0Marker.from_stream(stream, marker_code, offset)

        _App0Marker__init_.assert_called_once_with(
            ANY, marker_code, offset, length, density_units, x_density, y_density
        )
        assert isinstance(app0_marker, _App0Marker)

    def it_knows_the_image_dpi(self, dpi_fixture):
        density_units, x_density, y_density, horz_dpi, vert_dpi = dpi_fixture
        app0 = _App0Marker(
            None, None, None, density_units, x_density, y_density
        )
        assert app0.horz_dpi == horz_dpi
        assert app0.vert_dpi == vert_dpi

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def _App0Marker__init_(self, request):
        return initializer_mock(request, _App0Marker)

    @pytest.fixture(params=[
        (0, 100, 200,  72,  72),
        (1, 100, 200, 100, 200),
        (2, 100, 200, 254, 508),
    ])
    def dpi_fixture(self, request):
        density_units, x_density, y_density, horz_dpi, vert_dpi = (
            request.param
        )
        return density_units, x_density, y_density, horz_dpi, vert_dpi


class Describe_App1Marker(object):

    def it_can_construct_from_a_stream_and_offset(
        self, _App1Marker__init_, _tiff_from_exif_segment_
    ):
        bytes_ = b'\x00\x42Exif\x00\x00'
        marker_code, offset, length = JPEG_MARKER_CODE.APP1, 0, 66
        horz_dpi, vert_dpi = 42, 24
        stream = StreamReader(BytesIO(bytes_), BIG_ENDIAN)

        app1_marker = _App1Marker.from_stream(stream, marker_code, offset)

        _tiff_from_exif_segment_.assert_called_once_with(stream, offset, length)
        _App1Marker__init_.assert_called_once_with(
            ANY, marker_code, offset, length, horz_dpi, vert_dpi
        )
        assert isinstance(app1_marker, _App1Marker)

    def it_can_construct_from_non_Exif_APP1_segment(self, _App1Marker__init_):
        bytes_ = b'\x00\x42Foobar'
        marker_code, offset, length = JPEG_MARKER_CODE.APP1, 0, 66
        stream = StreamReader(BytesIO(bytes_), BIG_ENDIAN)

        app1_marker = _App1Marker.from_stream(stream, marker_code, offset)

        _App1Marker__init_.assert_called_once_with(
            ANY, marker_code, offset, length, 72, 72
        )
        assert isinstance(app1_marker, _App1Marker)

    def it_gets_a_tiff_from_its_Exif_segment_to_help_construct(
            self, get_tiff_fixture):
        stream, offset, length = get_tiff_fixture[:3]
        BytesIO_, segment_bytes, substream_ = get_tiff_fixture[3:6]
        Tiff_, tiff_ = get_tiff_fixture[6:]
        tiff = _App1Marker._tiff_from_exif_segment(stream, offset, length)
        BytesIO_.assert_called_once_with(segment_bytes)
        Tiff_.from_stream.assert_called_once_with(substream_)
        assert tiff is tiff_

    def it_knows_the_image_dpi(self):
        horz_dpi, vert_dpi = 42, 24
        app1 = _App1Marker(None, None, None, horz_dpi, vert_dpi)
        assert app1.horz_dpi == horz_dpi
        assert app1.vert_dpi == vert_dpi

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def _App1Marker__init_(self, request):
        return initializer_mock(request, _App1Marker)

    @pytest.fixture
    def BytesIO_(self, request, substream_):
        return class_mock(
            request, 'docx.image.jpeg.BytesIO', return_value=substream_
        )

    @pytest.fixture
    def get_tiff_fixture(self, request, BytesIO_, substream_, Tiff_, tiff_):
        bytes_ = b'xfillerxMM\x00*\x00\x00\x00\x42'
        stream_reader = StreamReader(BytesIO(bytes_), BIG_ENDIAN)
        offset, segment_length, segment_bytes = 0, 16, bytes_[8:]
        return (
            stream_reader, offset, segment_length, BytesIO_, segment_bytes,
            substream_, Tiff_, tiff_
        )

    @pytest.fixture
    def substream_(self, request):
        return instance_mock(request, BytesIO)

    @pytest.fixture
    def Tiff_(self, request, tiff_):
        Tiff_ = class_mock(request, 'docx.image.jpeg.Tiff')
        Tiff_.from_stream.return_value = tiff_
        return Tiff_

    @pytest.fixture
    def tiff_(self, request):
        return instance_mock(request, Tiff, horz_dpi=42, vert_dpi=24)

    @pytest.fixture
    def _tiff_from_exif_segment_(self, request, tiff_):
        return method_mock(
            request, _App1Marker, '_tiff_from_exif_segment', autospec=False,
            return_value=tiff_
        )


class Describe_SofMarker(object):

    def it_can_construct_from_a_stream_and_offset(self, request, _SofMarker__init_):
        bytes_ = b'\x00\x11\x00\x00\x2A\x00\x18'
        marker_code, offset, length = JPEG_MARKER_CODE.SOF0, 0, 17
        px_width, px_height = 24, 42
        stream = StreamReader(BytesIO(bytes_), BIG_ENDIAN)

        sof_marker = _SofMarker.from_stream(stream, marker_code, offset)

        _SofMarker__init_.assert_called_once_with(
            ANY, marker_code, offset, length, px_width, px_height
        )
        assert isinstance(sof_marker, _SofMarker)

    def it_knows_the_image_width_and_height(self):
        sof = _SofMarker(None, None, None, 42, 24)
        assert sof.px_width == 42
        assert sof.px_height == 24

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def _SofMarker__init_(self, request):
        return initializer_mock(request, _SofMarker)


class Describe_MarkerFactory(object):

    def it_constructs_the_appropriate_marker_object(self, call_fixture):
        marker_code, stream_, offset_, marker_cls_ = call_fixture
        marker = _MarkerFactory(marker_code, stream_, offset_)
        marker_cls_.from_stream.assert_called_once_with(
            stream_, marker_code, offset_
        )
        assert marker is marker_cls_.from_stream.return_value

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        JPEG_MARKER_CODE.APP0,
        JPEG_MARKER_CODE.APP1,
        JPEG_MARKER_CODE.SOF0,
        JPEG_MARKER_CODE.SOF7,
        JPEG_MARKER_CODE.SOS,
    ])
    def call_fixture(
            self, request, stream_, offset_, _App0Marker_, _App1Marker_,
            _SofMarker_, _Marker_):
        marker_code = request.param
        if marker_code == JPEG_MARKER_CODE.APP0:
            marker_cls_ = _App0Marker_
        elif marker_code == JPEG_MARKER_CODE.APP1:
            marker_cls_ = _App1Marker_
        elif marker_code in JPEG_MARKER_CODE.SOF_MARKER_CODES:
            marker_cls_ = _SofMarker_
        else:
            marker_cls_ = _Marker_
        return marker_code, stream_, offset_, marker_cls_

    @pytest.fixture
    def _App0Marker_(self, request):
        return class_mock(request, 'docx.image.jpeg._App0Marker')

    @pytest.fixture
    def _App1Marker_(self, request):
        return class_mock(request, 'docx.image.jpeg._App1Marker')

    @pytest.fixture
    def _Marker_(self, request):
        return class_mock(request, 'docx.image.jpeg._Marker')

    @pytest.fixture
    def offset_(self, request):
        return instance_mock(request, int)

    @pytest.fixture
    def _SofMarker_(self, request):
        return class_mock(request, 'docx.image.jpeg._SofMarker')

    @pytest.fixture
    def stream_(self, request):
        return instance_mock(request, BytesIO)


class Describe_MarkerFinder(object):

    def it_can_construct_from_a_stream(self, stream_, _MarkerFinder__init_):
        marker_finder = _MarkerFinder.from_stream(stream_)

        _MarkerFinder__init_.assert_called_once_with(ANY, stream_)
        assert isinstance(marker_finder, _MarkerFinder)

    def it_can_find_the_next_marker_after_a_given_offset(self, next_fixture):
        marker_finder, start, expected_code_and_offset = next_fixture
        marker_code, segment_offset = marker_finder.next(start)
        assert (marker_code, segment_offset) == expected_code_and_offset

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def _MarkerFinder__init_(self, request):
        return initializer_mock(request, _MarkerFinder)

    @pytest.fixture(params=[
        (0, JPEG_MARKER_CODE.SOI,   2),
        (1, JPEG_MARKER_CODE.APP0,  4),
        (2, JPEG_MARKER_CODE.APP0,  4),
        (3, JPEG_MARKER_CODE.EOI,  12),
        (4, JPEG_MARKER_CODE.EOI,  12),
        (6, JPEG_MARKER_CODE.EOI,  12),
        (8, JPEG_MARKER_CODE.EOI,  12),
    ])
    def next_fixture(self, request):
        start, marker_code, segment_offset = request.param
        bytes_ = b'\xFF\xD8\xFF\xE0\x00\x01\xFF\x00\xFF\xFF\xFF\xD9'
        stream_reader = StreamReader(BytesIO(bytes_), BIG_ENDIAN)
        marker_finder = _MarkerFinder(stream_reader)
        expected_code_and_offset = (marker_code, segment_offset)
        return marker_finder, start, expected_code_and_offset

    @pytest.fixture
    def stream_(self, request):
        return instance_mock(request, BytesIO)


class Describe_MarkerParser(object):

    def it_can_construct_from_a_jfif_stream(
        self, stream_, StreamReader_, _MarkerParser__init_, stream_reader_
    ):
        marker_parser = _MarkerParser.from_stream(stream_)

        StreamReader_.assert_called_once_with(stream_, BIG_ENDIAN)
        _MarkerParser__init_.assert_called_once_with(ANY, stream_reader_)
        assert isinstance(marker_parser, _MarkerParser)

    def it_can_iterate_over_the_jfif_markers_in_its_stream(
            self, iter_markers_fixture):
        (marker_parser, stream_, _MarkerFinder_, marker_finder_,
         _MarkerFactory_, marker_codes, offsets,
         marker_lst) = iter_markers_fixture
        markers = [marker for marker in marker_parser.iter_markers()]
        _MarkerFinder_.from_stream.assert_called_once_with(stream_)
        assert marker_finder_.next.call_args_list == [
            call(0), call(2), call(20)
        ]
        assert _MarkerFactory_.call_args_list == [
            call(marker_codes[0], stream_, offsets[0]),
            call(marker_codes[1], stream_, offsets[1]),
            call(marker_codes[2], stream_, offsets[2]),
        ]
        assert markers == marker_lst

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def app0_(self, request):
        return instance_mock(request, _App0Marker, segment_length=16)

    @pytest.fixture
    def eoi_(self, request):
        return instance_mock(request, _Marker, segment_length=0)

    @pytest.fixture
    def iter_markers_fixture(
            self, stream_reader_, _MarkerFinder_, marker_finder_,
            _MarkerFactory_, soi_, app0_, eoi_):
        marker_parser = _MarkerParser(stream_reader_)
        offsets = [2, 4, 22]
        marker_lst = [soi_, app0_, eoi_]
        marker_finder_.next.side_effect = [
            (JPEG_MARKER_CODE.SOI,  offsets[0]),
            (JPEG_MARKER_CODE.APP0, offsets[1]),
            (JPEG_MARKER_CODE.EOI,  offsets[2]),
        ]
        marker_codes = [
            JPEG_MARKER_CODE.SOI, JPEG_MARKER_CODE.APP0, JPEG_MARKER_CODE.EOI
        ]
        return (
            marker_parser, stream_reader_, _MarkerFinder_, marker_finder_,
            _MarkerFactory_, marker_codes, offsets, marker_lst
        )

    @pytest.fixture
    def _MarkerFactory_(self, request, soi_, app0_, eoi_):
        return class_mock(
            request, 'docx.image.jpeg._MarkerFactory',
            side_effect=[soi_, app0_, eoi_]
        )

    @pytest.fixture
    def _MarkerFinder_(self, request, marker_finder_):
        _MarkerFinder_ = class_mock(request, 'docx.image.jpeg._MarkerFinder')
        _MarkerFinder_.from_stream.return_value = marker_finder_
        return _MarkerFinder_

    @pytest.fixture
    def marker_finder_(self, request):
        return instance_mock(request, _MarkerFinder)

    @pytest.fixture
    def _MarkerParser__init_(self, request):
        return initializer_mock(request, _MarkerParser)

    @pytest.fixture
    def soi_(self, request):
        return instance_mock(request, _Marker, segment_length=0)

    @pytest.fixture
    def stream_(self, request):
        return instance_mock(request, BytesIO)

    @pytest.fixture
    def StreamReader_(self, request, stream_reader_):
        return class_mock(
            request, 'docx.image.jpeg.StreamReader',
            return_value=stream_reader_
        )

    @pytest.fixture
    def stream_reader_(self, request):
        return instance_mock(request, StreamReader)



# encoding: utf-8

"""
Test suite for opc.oxml module
"""

from docx.opc.constants import RELATIONSHIP_TARGET_MODE as RTM
from docx.opc.oxml import (
    CT_Default, CT_Override, CT_Relationship, CT_Relationships, CT_Types
)
from docx.oxml.xmlchemy import serialize_for_reading

from .unitdata.rels import (
    a_Default, an_Override, a_Relationship, a_Relationships, a_Types
)


class DescribeCT_Default(object):

    def it_provides_read_access_to_xml_values(self):
        default = a_Default().element
        assert default.extension == 'xml'
        assert default.content_type == 'application/xml'

    def it_can_construct_a_new_default_element(self):
        default = CT_Default.new('xml', 'application/xml')
        expected_xml = a_Default().xml
        assert default.xml == expected_xml


class DescribeCT_Override(object):

    def it_provides_read_access_to_xml_values(self):
        override = an_Override().element
        assert override.partname == '/part/name.xml'
        assert override.content_type == 'app/vnd.type'

    def it_can_construct_a_new_override_element(self):
        override = CT_Override.new('/part/name.xml', 'app/vnd.type')
        expected_xml = an_Override().xml
        assert override.xml == expected_xml


class DescribeCT_Relationship(object):

    def it_provides_read_access_to_xml_values(self):
        rel = a_Relationship().element
        assert rel.rId == 'rId9'
        assert rel.reltype == 'ReLtYpE'
        assert rel.target_ref == 'docProps/core.xml'
        assert rel.target_mode == RTM.INTERNAL

    def it_can_construct_from_attribute_values(self):
        cases = (
            ('rId9', 'ReLtYpE', 'foo/bar.xml',      None),
            ('rId9', 'ReLtYpE', 'bar/foo.xml',      RTM.INTERNAL),
            ('rId9', 'ReLtYpE', 'http://some/link', RTM.EXTERNAL),
        )
        for rId, reltype, target, target_mode in cases:
            if target_mode is None:
                rel = CT_Relationship.new(rId, reltype, target)
            else:
                rel = CT_Relationship.new(rId, reltype, target, target_mode)
            builder = a_Relationship().with_target(target)
            if target_mode == RTM.EXTERNAL:
                builder = builder.with_target_mode(RTM.EXTERNAL)
            expected_rel_xml = builder.xml
            assert rel.xml == expected_rel_xml


class DescribeCT_Relationships(object):

    def it_can_construct_a_new_relationships_element(self):
        rels = CT_Relationships.new()
        expected_xml = (
            '<Relationships xmlns="http://schemas.openxmlformats.org/package'
            '/2006/relationships"/>\n'
        )
        assert serialize_for_reading(rels) == expected_xml

    def it_can_build_rels_element_incrementally(self):
        # setup ------------------------
        rels = CT_Relationships.new()
        # exercise ---------------------
        rels.add_rel('rId1', 'http://reltype1', 'docProps/core.xml')
        rels.add_rel('rId2', 'http://linktype', 'http://some/link', True)
        rels.add_rel('rId3', 'http://reltype2', '../slides/slide1.xml')
        # verify -----------------------
        expected_rels_xml = a_Relationships().xml
        assert serialize_for_reading(rels) == expected_rels_xml

    def it_can_generate_rels_file_xml(self):
        expected_xml = (
            '<?xml version=\'1.0\' encoding=\'UTF-8\' standalone=\'yes\'?>\n'
            '<Relationships xmlns="http://schemas.openxmlformats.org/package'
            '/2006/relationships"/>'.encode('utf-8')
        )
        assert CT_Relationships.new().xml == expected_xml


class DescribeCT_Types(object):

    def it_provides_access_to_default_child_elements(self):
        types = a_Types().element
        assert len(types.defaults) == 2
        for default in types.defaults:
            assert isinstance(default, CT_Default)

    def it_provides_access_to_override_child_elements(self):
        types = a_Types().element
        assert len(types.overrides) == 3
        for override in types.overrides:
            assert isinstance(override, CT_Override)

    def it_should_have_empty_list_on_no_matching_elements(self):
        types = a_Types().empty().element
        assert types.defaults == []
        assert types.overrides == []

    def it_can_construct_a_new_types_element(self):
        types = CT_Types.new()
        expected_xml = a_Types().empty().xml
        assert types.xml == expected_xml

    def it_can_build_types_element_incrementally(self):
        types = CT_Types.new()
        types.add_default('xml', 'application/xml')
        types.add_default('jpeg', 'image/jpeg')
        types.add_override('/docProps/core.xml', 'app/vnd.type1')
        types.add_override('/ppt/presentation.xml', 'app/vnd.type2')
        types.add_override('/docProps/thumbnail.jpeg', 'image/jpeg')
        expected_types_xml = a_Types().xml
        assert types.xml == expected_types_xml



# encoding: utf-8

"""
Test suite for docx.image.helpers module
"""

from __future__ import absolute_import, print_function

import pytest

from docx.compat import BytesIO
from docx.image.exceptions import UnexpectedEndOfFileError
from docx.image.helpers import BIG_ENDIAN, LITTLE_ENDIAN, StreamReader


class DescribeStreamReader(object):

    def it_can_read_a_string_of_specified_len_at_offset(
            self, read_str_fixture):
        stream_rdr, expected_string = read_str_fixture
        s = stream_rdr.read_str(6, 2)
        assert s == 'foobar'

    def it_raises_on_unexpected_EOF(self, read_str_fixture):
        stream_rdr = read_str_fixture[0]
        with pytest.raises(UnexpectedEndOfFileError):
            stream_rdr.read_str(9, 2)

    def it_can_read_a_long(self, read_long_fixture):
        stream_rdr, offset, expected_int = read_long_fixture
        long_ = stream_rdr.read_long(offset)
        assert long_ == expected_int

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        (BIG_ENDIAN,    b'\xBE\x00\x00\x00\x2A\xEF', 1, 42),
        (LITTLE_ENDIAN, b'\xBE\xEF\x2A\x00\x00\x00', 2, 42),
    ])
    def read_long_fixture(self, request):
        byte_order, bytes_, offset, expected_int = request.param
        stream = BytesIO(bytes_)
        stream_rdr = StreamReader(stream, byte_order)
        return stream_rdr, offset, expected_int

    @pytest.fixture
    def read_str_fixture(self):
        stream = BytesIO(b'\x01\x02foobar\x03\x04')
        stream_rdr = StreamReader(stream, BIG_ENDIAN)
        expected_string = 'foobar'
        return stream_rdr, expected_string



# encoding: utf-8

"""
Helper methods and variables for acceptance tests.
"""

import os


def absjoin(*paths):
    return os.path.abspath(os.path.join(*paths))


thisdir = os.path.split(__file__)[0]
scratch_dir = absjoin(thisdir, '../_scratch')

# scratch output docx file -------------
saved_docx_path = absjoin(scratch_dir, 'test_out.docx')

bool_vals = {
    'True':  True,
    'False': False
}

test_text = 'python-docx was here!'

tri_state_vals = {
    'True':  True,
    'False': False,
    'None':  None,
}


def test_docx(name):
    """
    Return the absolute path to test .docx file with root name *name*.
    """
    return absjoin(thisdir, 'test_files', '%s.docx' % name)


def test_file(name):
    """
    Return the absolute path to file with *name* in test_files directory
    """
    return absjoin(thisdir, 'test_files', '%s' % name)



# encoding: utf-8

"""
Exceptions specific the the image sub-package
"""


class InvalidImageStreamError(Exception):
    """
    The recognized image stream appears to be corrupted
    """


class UnexpectedEndOfFileError(Exception):
    """
    EOF was unexpectedly encountered while reading an image stream.
    """


class UnrecognizedImageError(Exception):
    """
    The provided image stream could not be recognized.
    """



# encoding: utf-8

"""
Step implementations for paragraph format-related features.
"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

from behave import given, then, when

from docx import Document
from docx.enum.text import WD_ALIGN_PARAGRAPH, WD_LINE_SPACING
from docx.shared import Pt
from docx.text.tabstops import TabStops

from helpers import test_docx


# given ===================================================

@given('a paragraph format')
def given_a_paragraph_format(context):
    document = Document(test_docx('tab-stops'))
    context.paragraph_format = document.paragraphs[0].paragraph_format


@given('a paragraph format having {prop_name} set {setting}')
def given_a_paragraph_format_having_prop_set(context, prop_name, setting):
    style_name = {
        'to inherit': 'Normal',
        'On':         'Base',
        'Off':        'Citation',
    }[setting]
    document = Document(test_docx('sty-known-styles'))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given('a paragraph format having {setting} line spacing')
def given_a_paragraph_format_having_setting_line_spacing(context, setting):
    style_name = {
        'inherited': 'Normal',
        '14 pt':     'Base',
        'double':    'Citation',
    }[setting]
    document = Document(test_docx('sty-known-styles'))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given('a paragraph format having {setting} space {side}')
def given_a_paragraph_format_having_setting_spacing(context, setting, side):
    style_name = 'Normal' if setting == 'inherited' else 'Base'
    document = Document(test_docx('sty-known-styles'))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given('a paragraph format having {type} alignment')
def given_a_paragraph_format_having_align_type_alignment(context, type):
    style_name = {
        'inherited': 'Normal',
        'center':    'Base',
        'right':     'Citation',
    }[type]
    document = Document(test_docx('sty-known-styles'))
    context.paragraph_format = document.styles[style_name].paragraph_format


@given('a paragraph format having {type} indent of {value}')
def given_a_paragraph_format_having_type_indent_value(context, type, value):
    style_name = {
        'inherit':  'Normal',
        '18 pt':    'Base',
        '17.3 pt':  'Base',
        '-17.3 pt': 'Citation',
        '46.1 pt':  'Citation',
    }[value]
    document = Document(test_docx('sty-known-styles'))
    context.paragraph_format = document.styles[style_name].paragraph_format


# when ====================================================

@when('I assign {value} to paragraph_format.line_spacing')
def when_I_assign_value_to_paragraph_format_line_spacing(context, value):
    new_value = {
        'Pt(14)': Pt(14),
        '2':      2,
    }.get(value)
    new_value = float(value) if new_value is None else new_value
    context.paragraph_format.line_spacing = new_value


@when('I assign {value} to paragraph_format.line_spacing_rule')
def when_I_assign_value_to_paragraph_format_line_rule(context, value):
    new_value = {
        'None':                           None,
        'WD_LINE_SPACING.EXACTLY':        WD_LINE_SPACING.EXACTLY,
        'WD_LINE_SPACING.MULTIPLE':       WD_LINE_SPACING.MULTIPLE,
        'WD_LINE_SPACING.SINGLE':         WD_LINE_SPACING.SINGLE,
        'WD_LINE_SPACING.DOUBLE':         WD_LINE_SPACING.DOUBLE,
        'WD_LINE_SPACING.AT_LEAST':       WD_LINE_SPACING.AT_LEAST,
        'WD_LINE_SPACING.ONE_POINT_FIVE': WD_LINE_SPACING.ONE_POINT_FIVE,
    }[value]
    paragraph_format = context.paragraph_format
    paragraph_format.line_spacing_rule = new_value


@when('I assign {value} to paragraph_format.alignment')
def when_I_assign_value_to_paragraph_format_alignment(context, value):
    new_value = {
        'None':                      None,
        'WD_ALIGN_PARAGRAPH.CENTER': WD_ALIGN_PARAGRAPH.CENTER,
        'WD_ALIGN_PARAGRAPH.RIGHT':  WD_ALIGN_PARAGRAPH.RIGHT,
    }[value]
    paragraph_format = context.paragraph_format
    paragraph_format.alignment = new_value


@when('I assign {value} to paragraph_format.space_{side}')
def when_I_assign_value_to_paragraph_format_space(context, value, side):
    paragraph_format = context.paragraph_format
    prop_name = 'space_%s' % side
    new_value = {
        'None':   None,
        'Pt(12)': Pt(12),
        'Pt(18)': Pt(18),
    }[value]
    setattr(paragraph_format, prop_name, new_value)


@when('I assign {value} to paragraph_format.{type_}_indent')
def when_I_assign_value_to_paragraph_format_indent(context, value, type_):
    paragraph_format = context.paragraph_format
    prop_name = '%s_indent' % type_
    value = None if value == 'None' else Pt(float(value.split()[0]))
    setattr(paragraph_format, prop_name, value)


@when('I assign {value} to paragraph_format.{prop_name}')
def when_I_assign_value_to_paragraph_format_prop(context, value, prop_name):
    paragraph_format = context.paragraph_format
    value = {'None': None, 'True': True, 'False': False}[value]
    setattr(paragraph_format, prop_name, value)


# then =====================================================

@then('paragraph_format.tab_stops is a TabStops object')
def then_paragraph_format_tab_stops_is_a_tabstops_object(context):
    tab_stops = context.paragraph_format.tab_stops
    assert isinstance(tab_stops, TabStops)


@then('paragraph_format.alignment is {value}')
def then_paragraph_format_alignment_is_value(context, value):
    expected_value = {
        'None':                      None,
        'WD_ALIGN_PARAGRAPH.LEFT':   WD_ALIGN_PARAGRAPH.LEFT,
        'WD_ALIGN_PARAGRAPH.CENTER': WD_ALIGN_PARAGRAPH.CENTER,
        'WD_ALIGN_PARAGRAPH.RIGHT':  WD_ALIGN_PARAGRAPH.RIGHT,
    }[value]
    paragraph_format = context.paragraph_format
    assert paragraph_format.alignment == expected_value


@then('paragraph_format.line_spacing is {value}')
def then_paragraph_format_line_spacing_is_value(context, value):
    expected_value = (
        None if value == 'None' else
        float(value) if '.' in value else
        int(value)
    )
    paragraph_format = context.paragraph_format

    if expected_value is None or isinstance(expected_value, int):
        assert paragraph_format.line_spacing == expected_value
    else:
        assert abs(paragraph_format.line_spacing - expected_value) < 0.001


@then('paragraph_format.line_spacing_rule is {value}')
def then_paragraph_format_line_spacing_rule_is_value(context, value):
    expected_value = {
        'None':                           None,
        'WD_LINE_SPACING.EXACTLY':        WD_LINE_SPACING.EXACTLY,
        'WD_LINE_SPACING.MULTIPLE':       WD_LINE_SPACING.MULTIPLE,
        'WD_LINE_SPACING.SINGLE':         WD_LINE_SPACING.SINGLE,
        'WD_LINE_SPACING.DOUBLE':         WD_LINE_SPACING.DOUBLE,
        'WD_LINE_SPACING.AT_LEAST':       WD_LINE_SPACING.AT_LEAST,
        'WD_LINE_SPACING.ONE_POINT_FIVE': WD_LINE_SPACING.ONE_POINT_FIVE,
    }[value]
    paragraph_format = context.paragraph_format
    assert paragraph_format.line_spacing_rule == expected_value


@then('paragraph_format.space_{side} is {value}')
def then_paragraph_format_space_side_is_value(context, side, value):
    expected_value = None if value == 'None' else int(value)
    prop_name = 'space_%s' % side
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value


@then('paragraph_format.{type_}_indent is {value}')
def then_paragraph_format_type_indent_is_value(context, type_, value):
    expected_value = None if value == 'None' else int(value)
    prop_name = '%s_indent' % type_
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value


@then('paragraph_format.{prop_name} is {value}')
def then_paragraph_format_prop_name_is_value(context, prop_name, value):
    expected_value = {'None': None, 'True': True, 'False': False}[value]
    paragraph_format = context.paragraph_format
    actual_value = getattr(paragraph_format, prop_name)
    assert actual_value == expected_value



# encoding: utf-8

"""
Enumerations used by DrawingML objects
"""

from __future__ import absolute_import

from .base import (
    alias, Enumeration, EnumMember, XmlEnumeration, XmlMappedEnumMember
)


class MSO_COLOR_TYPE(Enumeration):
    """
    Specifies the color specification scheme

    Example::

        from docx.enum.dml import MSO_COLOR_TYPE

        assert font.color.type == MSO_COLOR_TYPE.SCHEME
    """

    __ms_name__ = 'MsoColorType'

    __url__ = (
        'http://msdn.microsoft.com/en-us/library/office/ff864912(v=office.15'
        ').aspx'
    )

    __members__ = (
        EnumMember(
            'RGB', 1, 'Color is specified by an |RGBColor| value.'
        ),
        EnumMember(
            'THEME', 2, 'Color is one of the preset theme colors.'
        ),
        EnumMember(
            'AUTO', 101, 'Color is determined automatically by the '
            'application.'
        ),
    )


@alias('MSO_THEME_COLOR')
class MSO_THEME_COLOR_INDEX(XmlEnumeration):
    """
    Indicates the Office theme color, one of those shown in the color gallery
    on the formatting ribbon.

    Alias: ``MSO_THEME_COLOR``

    Example::

        from docx.enum.dml import MSO_THEME_COLOR

        font.color.theme_color = MSO_THEME_COLOR.ACCENT_1
    """

    __ms_name__ = 'MsoThemeColorIndex'

    __url__ = (
        'http://msdn.microsoft.com/en-us/library/office/ff860782(v=office.15'
        ').aspx'
    )

    __members__ = (
        EnumMember(
            'NOT_THEME_COLOR', 0, 'Indicates the color is not a theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_1', 5, 'accent1', 'Specifies the Accent 1 theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_2', 6, 'accent2', 'Specifies the Accent 2 theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_3', 7, 'accent3', 'Specifies the Accent 3 theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_4', 8, 'accent4', 'Specifies the Accent 4 theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_5', 9, 'accent5', 'Specifies the Accent 5 theme color.'
        ),
        XmlMappedEnumMember(
            'ACCENT_6', 10, 'accent6', 'Specifies the Accent 6 theme color.'
        ),
        XmlMappedEnumMember(
            'BACKGROUND_1', 14, 'background1', 'Specifies the Background 1 '
            'theme color.'
        ),
        XmlMappedEnumMember(
            'BACKGROUND_2', 16, 'background2', 'Specifies the Background 2 '
            'theme color.'
        ),
        XmlMappedEnumMember(
            'DARK_1', 1, 'dark1', 'Specifies the Dark 1 theme color.'
        ),
        XmlMappedEnumMember(
            'DARK_2', 3, 'dark2', 'Specifies the Dark 2 theme color.'
        ),
        XmlMappedEnumMember(
            'FOLLOWED_HYPERLINK', 12, 'followedHyperlink', 'Specifies the '
            'theme color for a clicked hyperlink.'
        ),
        XmlMappedEnumMember(
            'HYPERLINK', 11, 'hyperlink', 'Specifies the theme color for a '
            'hyperlink.'
        ),
        XmlMappedEnumMember(
            'LIGHT_1', 2, 'light1', 'Specifies the Light 1 theme color.'
        ),
        XmlMappedEnumMember(
            'LIGHT_2', 4, 'light2', 'Specifies the Light 2 theme color.'
        ),
        XmlMappedEnumMember(
            'TEXT_1', 13, 'text1', 'Specifies the Text 1 theme color.'
        ),
        XmlMappedEnumMember(
            'TEXT_2', 15, 'text2', 'Specifies the Text 2 theme color.'
        ),
    )



# encoding: utf-8

"""
Test suite for the docx.oxml.parts module.
"""

from __future__ import absolute_import, print_function, unicode_literals

import pytest

from ...unitutil.cxml import element, xml


class DescribeCT_Body(object):

    def it_can_clear_all_its_content(self, clear_fixture):
        body, expected_xml = clear_fixture
        body.clear_content()
        assert body.xml == expected_xml

    def it_can_add_a_section_break(self, section_break_fixture):
        body, expected_xml = section_break_fixture
        sectPr = body.add_section_break()
        assert body.xml == expected_xml
        assert sectPr is body.get_or_add_sectPr()

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        ('w:body',                 'w:body'),
        ('w:body/w:p',             'w:body'),
        ('w:body/w:tbl',           'w:body'),
        ('w:body/w:sectPr',        'w:body/w:sectPr'),
        ('w:body/(w:p, w:sectPr)', 'w:body/w:sectPr'),
    ])
    def clear_fixture(self, request):
        before_cxml, after_cxml = request.param
        body = element(before_cxml)
        expected_xml = xml(after_cxml)
        return body, expected_xml

    @pytest.fixture
    def section_break_fixture(self):
        body = element('w:body/w:sectPr/w:type{w:val=foobar}')
        expected_xml = xml(
            'w:body/('
            '  w:p/w:pPr/w:sectPr/w:type{w:val=foobar},'
            '  w:sectPr/w:type{w:val=foobar}'
            ')'
        )
        return body, expected_xml



# encoding: utf-8

"""
Test data builders for styles part XML elements
"""

from ...unitdata import BaseBuilder


class CT_StyleBuilder(BaseBuilder):
    __tag__ = 'w:style'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:type', 'w:styleId', 'w:default', 'w:customStyle')


class CT_StylesBuilder(BaseBuilder):
    __tag__ = 'w:styles'
    __nspfxs__ = ('w',)
    __attrs__ = ()


def a_style():
    return CT_StyleBuilder()


def a_styles():
    return CT_StylesBuilder()



# encoding: utf-8

"""
Enumerations related to styles
"""

from __future__ import absolute_import, print_function, unicode_literals

from .base import alias, EnumMember, XmlEnumeration, XmlMappedEnumMember


@alias('WD_STYLE')
class WD_BUILTIN_STYLE(XmlEnumeration):
    """
    alias: **WD_STYLE**

    Specifies a built-in Microsoft Word style.

    Example::

        from docx import Document
        from docx.enum.style import WD_STYLE

        document = Document()
        styles = document.styles
        style = styles[WD_STYLE.BODY_TEXT]
    """

    __ms_name__ = 'WdBuiltinStyle'

    __url__ = 'http://msdn.microsoft.com/en-us/library/office/ff835210.aspx'

    __members__ = (
        EnumMember(
            'BLOCK_QUOTATION', -85, 'Block Text.'
        ),
        EnumMember(
            'BODY_TEXT', -67, 'Body Text.'
        ),
        EnumMember(
            'BODY_TEXT_2', -81, 'Body Text 2.'
        ),
        EnumMember(
            'BODY_TEXT_3', -82, 'Body Text 3.'
        ),
        EnumMember(
            'BODY_TEXT_FIRST_INDENT', -78, 'Body Text First Indent.'
        ),
        EnumMember(
            'BODY_TEXT_FIRST_INDENT_2', -79, 'Body Text First Indent 2.'
        ),
        EnumMember(
            'BODY_TEXT_INDENT', -68, 'Body Text Indent.'
        ),
        EnumMember(
            'BODY_TEXT_INDENT_2', -83, 'Body Text Indent 2.'
        ),
        EnumMember(
            'BODY_TEXT_INDENT_3', -84, 'Body Text Indent 3.'
        ),
        EnumMember(
            'BOOK_TITLE', -265, 'Book Title.'
        ),
        EnumMember(
            'CAPTION', -35, 'Caption.'
        ),
        EnumMember(
            'CLOSING', -64, 'Closing.'
        ),
        EnumMember(
            'COMMENT_REFERENCE', -40, 'Comment Reference.'
        ),
        EnumMember(
            'COMMENT_TEXT', -31, 'Comment Text.'
        ),
        EnumMember(
            'DATE', -77, 'Date.'
        ),
        EnumMember(
            'DEFAULT_PARAGRAPH_FONT', -66, 'Default Paragraph Font.'
        ),
        EnumMember(
            'EMPHASIS', -89, 'Emphasis.'
        ),
        EnumMember(
            'ENDNOTE_REFERENCE', -43, 'Endnote Reference.'
        ),
        EnumMember(
            'ENDNOTE_TEXT', -44, 'Endnote Text.'
        ),
        EnumMember(
            'ENVELOPE_ADDRESS', -37, 'Envelope Address.'
        ),
        EnumMember(
            'ENVELOPE_RETURN', -38, 'Envelope Return.'
        ),
        EnumMember(
            'FOOTER', -33, 'Footer.'
        ),
        EnumMember(
            'FOOTNOTE_REFERENCE', -39, 'Footnote Reference.'
        ),
        EnumMember(
            'FOOTNOTE_TEXT', -30, 'Footnote Text.'
        ),
        EnumMember(
            'HEADER', -32, 'Header.'
        ),
        EnumMember(
            'HEADING_1', -2, 'Heading 1.'
        ),
        EnumMember(
            'HEADING_2', -3, 'Heading 2.'
        ),
        EnumMember(
            'HEADING_3', -4, 'Heading 3.'
        ),
        EnumMember(
            'HEADING_4', -5, 'Heading 4.'
        ),
        EnumMember(
            'HEADING_5', -6, 'Heading 5.'
        ),
        EnumMember(
            'HEADING_6', -7, 'Heading 6.'
        ),
        EnumMember(
            'HEADING_7', -8, 'Heading 7.'
        ),
        EnumMember(
            'HEADING_8', -9, 'Heading 8.'
        ),
        EnumMember(
            'HEADING_9', -10, 'Heading 9.'
        ),
        EnumMember(
            'HTML_ACRONYM', -96, 'HTML Acronym.'
        ),
        EnumMember(
            'HTML_ADDRESS', -97, 'HTML Address.'
        ),
        EnumMember(
            'HTML_CITE', -98, 'HTML Cite.'
        ),
        EnumMember(
            'HTML_CODE', -99, 'HTML Code.'
        ),
        EnumMember(
            'HTML_DFN', -100, 'HTML Definition.'
        ),
        EnumMember(
            'HTML_KBD', -101, 'HTML Keyboard.'
        ),
        EnumMember(
            'HTML_NORMAL', -95, 'Normal (Web).'
        ),
        EnumMember(
            'HTML_PRE', -102, 'HTML Preformatted.'
        ),
        EnumMember(
            'HTML_SAMP', -103, 'HTML Sample.'
        ),
        EnumMember(
            'HTML_TT', -104, 'HTML Typewriter.'
        ),
        EnumMember(
            'HTML_VAR', -105, 'HTML Variable.'
        ),
        EnumMember(
            'HYPERLINK', -86, 'Hyperlink.'
        ),
        EnumMember(
            'HYPERLINK_FOLLOWED', -87, 'Followed Hyperlink.'
        ),
        EnumMember(
            'INDEX_1', -11, 'Index 1.'
        ),
        EnumMember(
            'INDEX_2', -12, 'Index 2.'
        ),
        EnumMember(
            'INDEX_3', -13, 'Index 3.'
        ),
        EnumMember(
            'INDEX_4', -14, 'Index 4.'
        ),
        EnumMember(
            'INDEX_5', -15, 'Index 5.'
        ),
        EnumMember(
            'INDEX_6', -16, 'Index 6.'
        ),
        EnumMember(
            'INDEX_7', -17, 'Index 7.'
        ),
        EnumMember(
            'INDEX_8', -18, 'Index 8.'
        ),
        EnumMember(
            'INDEX_9', -19, 'Index 9.'
        ),
        EnumMember(
            'INDEX_HEADING', -34, 'Index Heading'
        ),
        EnumMember(
            'INTENSE_EMPHASIS', -262, 'Intense Emphasis.'
        ),
        EnumMember(
            'INTENSE_QUOTE', -182, 'Intense Quote.'
        ),
        EnumMember(
            'INTENSE_REFERENCE', -264, 'Intense Reference.'
        ),
        EnumMember(
            'LINE_NUMBER', -41, 'Line Number.'
        ),
        EnumMember(
            'LIST', -48, 'List.'
        ),
        EnumMember(
            'LIST_2', -51, 'List 2.'
        ),
        EnumMember(
            'LIST_3', -52, 'List 3.'
        ),
        EnumMember(
            'LIST_4', -53, 'List 4.'
        ),
        EnumMember(
            'LIST_5', -54, 'List 5.'
        ),
        EnumMember(
            'LIST_BULLET', -49, 'List Bullet.'
        ),
        EnumMember(
            'LIST_BULLET_2', -55, 'List Bullet 2.'
        ),
        EnumMember(
            'LIST_BULLET_3', -56, 'List Bullet 3.'
        ),
        EnumMember(
            'LIST_BULLET_4', -57, 'List Bullet 4.'
        ),
        EnumMember(
            'LIST_BULLET_5', -58, 'List Bullet 5.'
        ),
        EnumMember(
            'LIST_CONTINUE', -69, 'List Continue.'
        ),
        EnumMember(
            'LIST_CONTINUE_2', -70, 'List Continue 2.'
        ),
        EnumMember(
            'LIST_CONTINUE_3', -71, 'List Continue 3.'
        ),
        EnumMember(
            'LIST_CONTINUE_4', -72, 'List Continue 4.'
        ),
        EnumMember(
            'LIST_CONTINUE_5', -73, 'List Continue 5.'
        ),
        EnumMember(
            'LIST_NUMBER', -50, 'List Number.'
        ),
        EnumMember(
            'LIST_NUMBER_2', -59, 'List Number 2.'
        ),
        EnumMember(
            'LIST_NUMBER_3', -60, 'List Number 3.'
        ),
        EnumMember(
            'LIST_NUMBER_4', -61, 'List Number 4.'
        ),
        EnumMember(
            'LIST_NUMBER_5', -62, 'List Number 5.'
        ),
        EnumMember(
            'LIST_PARAGRAPH', -180, 'List Paragraph.'
        ),
        EnumMember(
            'MACRO_TEXT', -46, 'Macro Text.'
        ),
        EnumMember(
            'MESSAGE_HEADER', -74, 'Message Header.'
        ),
        EnumMember(
            'NAV_PANE', -90, 'Document Map.'
        ),
        EnumMember(
            'NORMAL', -1, 'Normal.'
        ),
        EnumMember(
            'NORMAL_INDENT', -29, 'Normal Indent.'
        ),
        EnumMember(
            'NORMAL_OBJECT', -158, 'Normal (applied to an object).'
        ),
        EnumMember(
            'NORMAL_TABLE', -106, 'Normal (applied within a table).'
        ),
        EnumMember(
            'NOTE_HEADING', -80, 'Note Heading.'
        ),
        EnumMember(
            'PAGE_NUMBER', -42, 'Page Number.'
        ),
        EnumMember(
            'PLAIN_TEXT', -91, 'Plain Text.'
        ),
        EnumMember(
            'QUOTE', -181, 'Quote.'
        ),
        EnumMember(
            'SALUTATION', -76, 'Salutation.'
        ),
        EnumMember(
            'SIGNATURE', -65, 'Signature.'
        ),
        EnumMember(
            'STRONG', -88, 'Strong.'
        ),
        EnumMember(
            'SUBTITLE', -75, 'Subtitle.'
        ),
        EnumMember(
            'SUBTLE_EMPHASIS', -261, 'Subtle Emphasis.'
        ),
        EnumMember(
            'SUBTLE_REFERENCE', -263, 'Subtle Reference.'
        ),
        EnumMember(
            'TABLE_COLORFUL_GRID', -172, 'Colorful Grid.'
        ),
        EnumMember(
            'TABLE_COLORFUL_LIST', -171, 'Colorful List.'
        ),
        EnumMember(
            'TABLE_COLORFUL_SHADING', -170, 'Colorful Shading.'
        ),
        EnumMember(
            'TABLE_DARK_LIST', -169, 'Dark List.'
        ),
        EnumMember(
            'TABLE_LIGHT_GRID', -161, 'Light Grid.'
        ),
        EnumMember(
            'TABLE_LIGHT_GRID_ACCENT_1', -175, 'Light Grid Accent 1.'
        ),
        EnumMember(
            'TABLE_LIGHT_LIST', -160, 'Light List.'
        ),
        EnumMember(
            'TABLE_LIGHT_LIST_ACCENT_1', -174, 'Light List Accent 1.'
        ),
        EnumMember(
            'TABLE_LIGHT_SHADING', -159, 'Light Shading.'
        ),
        EnumMember(
            'TABLE_LIGHT_SHADING_ACCENT_1', -173, 'Light Shading Accent 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_GRID_1', -166, 'Medium Grid 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_GRID_2', -167, 'Medium Grid 2.'
        ),
        EnumMember(
            'TABLE_MEDIUM_GRID_3', -168, 'Medium Grid 3.'
        ),
        EnumMember(
            'TABLE_MEDIUM_LIST_1', -164, 'Medium List 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_LIST_1_ACCENT_1', -178, 'Medium List 1 Accent 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_LIST_2', -165, 'Medium List 2.'
        ),
        EnumMember(
            'TABLE_MEDIUM_SHADING_1', -162, 'Medium Shading 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_SHADING_1_ACCENT_1', -176,
            'Medium Shading 1 Accent 1.'
        ),
        EnumMember(
            'TABLE_MEDIUM_SHADING_2', -163, 'Medium Shading 2.'
        ),
        EnumMember(
            'TABLE_MEDIUM_SHADING_2_ACCENT_1', -177,
            'Medium Shading 2 Accent 1.'
        ),
        EnumMember(
            'TABLE_OF_AUTHORITIES', -45, 'Table of Authorities.'
        ),
        EnumMember(
            'TABLE_OF_FIGURES', -36, 'Table of Figures.'
        ),
        EnumMember(
            'TITLE', -63, 'Title.'
        ),
        EnumMember(
            'TOAHEADING', -47, 'TOA Heading.'
        ),
        EnumMember(
            'TOC_1', -20, 'TOC 1.'
        ),
        EnumMember(
            'TOC_2', -21, 'TOC 2.'
        ),
        EnumMember(
            'TOC_3', -22, 'TOC 3.'
        ),
        EnumMember(
            'TOC_4', -23, 'TOC 4.'
        ),
        EnumMember(
            'TOC_5', -24, 'TOC 5.'
        ),
        EnumMember(
            'TOC_6', -25, 'TOC 6.'
        ),
        EnumMember(
            'TOC_7', -26, 'TOC 7.'
        ),
        EnumMember(
            'TOC_8', -27, 'TOC 8.'
        ),
        EnumMember(
            'TOC_9', -28, 'TOC 9.'
        ),
    )


class WD_STYLE_TYPE(XmlEnumeration):
    """
    Specifies one of the four style types: paragraph, character, list, or
    table.

    Example::

        from docx import Document
        from docx.enum.style import WD_STYLE_TYPE

        styles = Document().styles
        assert styles[0].type == WD_STYLE_TYPE.PARAGRAPH
    """

    __ms_name__ = 'WdStyleType'

    __url__ = 'http://msdn.microsoft.com/en-us/library/office/ff196870.aspx'

    __members__ = (
        XmlMappedEnumMember(
            'CHARACTER', 2, 'character', 'Character style.'
        ),
        XmlMappedEnumMember(
            'LIST', 4, 'numbering', 'List style.'
        ),
        XmlMappedEnumMember(
            'PARAGRAPH', 1, 'paragraph', 'Paragraph style.'
        ),
        XmlMappedEnumMember(
            'TABLE', 3, 'table', 'Table style.'
        ),
    )



# encoding: utf-8

"""
Test suite for docx.enum module, focused on base classes. Configured a little
differently because of the meta-programming, the two enumeration classes at
the top constitute the entire fixture and the tests themselves just make
assertions on those.
"""

from __future__ import absolute_import, print_function

import pytest

from docx.enum.base import (
    alias, Enumeration, EnumMember, ReturnValueOnlyEnumMember,
    XmlEnumeration, XmlMappedEnumMember
)


@alias('BARFOO')
class FOOBAR(Enumeration):
    """
    Enumeration docstring
    """

    __ms_name__ = 'MsoFoobar'

    __url__ = 'http://msdn.microsoft.com/foobar.aspx'

    __members__ = (
        EnumMember(None, None, 'No setting/remove setting'),
        EnumMember('READ_WRITE', 1, 'Readable and settable'),
        ReturnValueOnlyEnumMember('READ_ONLY', -2, 'Return value only'),
    )


@alias('XML-FU')
class XMLFOO(XmlEnumeration):
    """
    XmlEnumeration docstring
    """

    __ms_name__ = 'MsoXmlFoobar'

    __url__ = 'http://msdn.microsoft.com/msoxmlfoobar.aspx'

    __members__ = (
        XmlMappedEnumMember(None, None, None, 'No setting'),
        XmlMappedEnumMember('XML_RW', 42, 'attrVal', 'Read/write setting'),
        ReturnValueOnlyEnumMember('RO', -2, 'Return value only;'),
    )


class DescribeEnumeration(object):

    def it_has_the_right_metaclass(self):
        assert type(FOOBAR).__name__ == 'MetaEnumeration'

    def it_provides_an_EnumValue_instance_for_each_named_member(self):
        with pytest.raises(AttributeError):
            getattr(FOOBAR, 'None')
        for obj in (FOOBAR.READ_WRITE, FOOBAR.READ_ONLY):
            assert type(obj).__name__ == 'EnumValue'

    def it_provides_the_enumeration_value_for_each_named_member(self):
        assert FOOBAR.READ_WRITE == 1
        assert FOOBAR.READ_ONLY == -2

    def it_knows_if_a_setting_is_valid(self):
        FOOBAR.validate(None)
        FOOBAR.validate(FOOBAR.READ_WRITE)
        with pytest.raises(ValueError):
            FOOBAR.validate('foobar')
        with pytest.raises(ValueError):
            FOOBAR.validate(FOOBAR.READ_ONLY)

    def it_can_be_referred_to_by_a_convenience_alias_if_defined(self):
        assert BARFOO is FOOBAR  # noqa


class DescribeEnumValue(object):

    def it_provides_its_symbolic_name_as_its_string_value(self):
        assert ('%s' % FOOBAR.READ_WRITE) == 'READ_WRITE (1)'

    def it_provides_its_description_as_its_docstring(self):
        assert FOOBAR.READ_ONLY.__doc__ == 'Return value only'


class DescribeXmlEnumeration(object):

    def it_knows_the_XML_value_for_each_of_its_xml_members(self):
        assert XMLFOO.to_xml(XMLFOO.XML_RW) == 'attrVal'
        assert XMLFOO.to_xml(42) == 'attrVal'
        with pytest.raises(ValueError):
            XMLFOO.to_xml(XMLFOO.RO)

    def it_can_map_each_of_its_xml_members_from_the_XML_value(self):
        assert XMLFOO.from_xml(None) is None
        assert XMLFOO.from_xml('attrVal') == XMLFOO.XML_RW
        assert str(XMLFOO.from_xml('attrVal')) == 'XML_RW (42)'



# encoding: utf-8

"""
Test suite for the docx.styles.style module
"""

from __future__ import (
    absolute_import, division, print_function, unicode_literals
)

import pytest

from docx.enum.style import WD_STYLE_TYPE
from docx.styles.style import (
    BaseStyle, _CharacterStyle, _ParagraphStyle, _NumberingStyle,
    StyleFactory, _TableStyle
)
from docx.text.font import Font
from docx.text.parfmt import ParagraphFormat

from ..unitutil.cxml import element, xml
from ..unitutil.mock import call, class_mock, function_mock, instance_mock


class DescribeStyleFactory(object):

    def it_constructs_the_right_type_of_style(self, factory_fixture):
        style_elm, StyleCls_, style_ = factory_fixture
        style = StyleFactory(style_elm)
        StyleCls_.assert_called_once_with(style_elm)
        assert style is style_

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=['paragraph', 'character', 'table', 'numbering'])
    def factory_fixture(
            self, request, paragraph_style_, _ParagraphStyle_,
            character_style_, _CharacterStyle_, table_style_, _TableStyle_,
            numbering_style_, _NumberingStyle_):
        type_attr_val = request.param
        StyleCls_, style_mock = {
            'paragraph': (_ParagraphStyle_, paragraph_style_),
            'character': (_CharacterStyle_, character_style_),
            'table':     (_TableStyle_,     table_style_),
            'numbering': (_NumberingStyle_, numbering_style_),
        }[request.param]
        style_cxml = 'w:style{w:type=%s}' % type_attr_val
        style_elm = element(style_cxml)
        return style_elm, StyleCls_, style_mock

    # fixture components -----------------------------------

    @pytest.fixture
    def _ParagraphStyle_(self, request, paragraph_style_):
        return class_mock(
            request, 'docx.styles.style._ParagraphStyle',
            return_value=paragraph_style_
        )

    @pytest.fixture
    def paragraph_style_(self, request):
        return instance_mock(request, _ParagraphStyle)

    @pytest.fixture
    def _CharacterStyle_(self, request, character_style_):
        return class_mock(
            request, 'docx.styles.style._CharacterStyle',
            return_value=character_style_
        )

    @pytest.fixture
    def character_style_(self, request):
        return instance_mock(request, _CharacterStyle)

    @pytest.fixture
    def _TableStyle_(self, request, table_style_):
        return class_mock(
            request, 'docx.styles.style._TableStyle',
            return_value=table_style_
        )

    @pytest.fixture
    def table_style_(self, request):
        return instance_mock(request, _TableStyle)

    @pytest.fixture
    def _NumberingStyle_(self, request, numbering_style_):
        return class_mock(
            request, 'docx.styles.style._NumberingStyle',
            return_value=numbering_style_
        )

    @pytest.fixture
    def numbering_style_(self, request):
        return instance_mock(request, _NumberingStyle)


class DescribeBaseStyle(object):

    def it_knows_its_style_id(self, id_get_fixture):
        style, expected_value = id_get_fixture
        assert style.style_id == expected_value

    def it_can_change_its_style_id(self, id_set_fixture):
        style, new_value, expected_xml = id_set_fixture
        style.style_id = new_value
        assert style._element.xml == expected_xml

    def it_knows_its_type(self, type_get_fixture):
        style, expected_value = type_get_fixture
        assert style.type == expected_value

    def it_knows_its_name(self, name_get_fixture):
        style, expected_value = name_get_fixture
        assert style.name == expected_value

    def it_can_change_its_name(self, name_set_fixture):
        style, new_value, expected_xml = name_set_fixture
        style.name = new_value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_a_builtin_style(self, builtin_get_fixture):
        style, expected_value = builtin_get_fixture
        assert style.builtin is expected_value

    def it_knows_whether_its_hidden(self, hidden_get_fixture):
        style, expected_value = hidden_get_fixture
        assert style.hidden == expected_value

    def it_can_change_whether_its_hidden(self, hidden_set_fixture):
        style, value, expected_xml = hidden_set_fixture
        style.hidden = value
        assert style._element.xml == expected_xml

    def it_knows_its_sort_order(self, priority_get_fixture):
        style, expected_value = priority_get_fixture
        assert style.priority == expected_value

    def it_can_change_its_sort_order(self, priority_set_fixture):
        style, value, expected_xml = priority_set_fixture
        style.priority = value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_unhide_when_used(self, unhide_get_fixture):
        style, expected_value = unhide_get_fixture
        assert style.unhide_when_used == expected_value

    def it_can_change_its_unhide_when_used_value(self, unhide_set_fixture):
        style, value, expected_xml = unhide_set_fixture
        style.unhide_when_used = value
        assert style._element.xml == expected_xml

    def it_knows_its_quick_style_setting(self, quick_get_fixture):
        style, expected_value = quick_get_fixture
        assert style.quick_style == expected_value

    def it_can_change_its_quick_style_setting(self, quick_set_fixture):
        style, new_value, expected_xml = quick_set_fixture
        style.quick_style = new_value
        assert style._element.xml == expected_xml

    def it_knows_whether_its_locked(self, locked_get_fixture):
        style, expected_value = locked_get_fixture
        assert style.locked == expected_value

    def it_can_change_whether_its_locked(self, locked_set_fixture):
        style, value, expected_xml = locked_set_fixture
        style.locked = value
        assert style._element.xml == expected_xml

    def it_can_delete_itself_from_the_document(self, delete_fixture):
        style, styles, expected_xml = delete_fixture
        style.delete()
        assert styles.xml == expected_xml
        assert style._element is None

    # fixture --------------------------------------------------------

    @pytest.fixture(params=[
        ('w:style',                  True),
        ('w:style{w:customStyle=0}', True),
        ('w:style{w:customStyle=1}', False),
    ])
    def builtin_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture
    def delete_fixture(self):
        styles = element('w:styles/w:style')
        style = BaseStyle(styles[0])
        expected_xml = xml('w:styles')
        return style, styles, expected_xml

    @pytest.fixture(params=[
        ('w:style',                       False),
        ('w:style/w:semiHidden',          True),
        ('w:style/w:semiHidden{w:val=0}', False),
        ('w:style/w:semiHidden{w:val=1}', True),
    ])
    def hidden_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                       True,  'w:style/w:semiHidden'),
        ('w:style/w:semiHidden{w:val=0}', True,  'w:style/w:semiHidden'),
        ('w:style/w:semiHidden{w:val=1}', True,  'w:style/w:semiHidden'),
        ('w:style',                       False, 'w:style'),
        ('w:style/w:semiHidden',          False, 'w:style'),
        ('w:style/w:semiHidden{w:val=1}', False, 'w:style'),
    ])
    def hidden_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(params=[
        ('w:style',                   None),
        ('w:style{w:styleId=Foobar}', 'Foobar'),
    ])
    def id_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                'Foo', 'w:style{w:styleId=Foo}'),
        ('w:style{w:styleId=Foo}', 'Bar', 'w:style{w:styleId=Bar}'),
        ('w:style{w:styleId=Bar}', None,  'w:style'),
        ('w:style',                None,  'w:style'),
    ])
    def id_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(params=[
        ('w:style',                   False),
        ('w:style/w:locked',          True),
        ('w:style/w:locked{w:val=0}', False),
        ('w:style/w:locked{w:val=1}', True),
    ])
    def locked_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                   True,  'w:style/w:locked'),
        ('w:style/w:locked{w:val=0}', True,  'w:style/w:locked'),
        ('w:style/w:locked{w:val=1}', True,  'w:style/w:locked'),
        ('w:style',                   False, 'w:style'),
        ('w:style/w:locked',          False, 'w:style'),
        ('w:style/w:locked{w:val=1}', False, 'w:style'),
    ])
    def locked_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(params=[
        ('w:style{w:type=table}',                         None),
        ('w:style{w:type=table}/w:name{w:val=Boofar}',    'Boofar'),
        ('w:style{w:type=table}/w:name{w:val=heading 1}', 'Heading 1'),
    ])
    def name_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                   'Foo', 'w:style/w:name{w:val=Foo}'),
        ('w:style/w:name{w:val=Foo}', 'Bar', 'w:style/w:name{w:val=Bar}'),
        ('w:style/w:name{w:val=Bar}', None,  'w:style'),
    ])
    def name_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(params=[
        ('w:style',                        None),
        ('w:style/w:uiPriority{w:val=42}', 42),
    ])
    def priority_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                        42,
         'w:style/w:uiPriority{w:val=42}'),
        ('w:style/w:uiPriority{w:val=42}', 24,
         'w:style/w:uiPriority{w:val=24}'),
        ('w:style/w:uiPriority{w:val=24}', None,
         'w:style'),
    ])
    def priority_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml

    @pytest.fixture(params=[
        ('w:style',                     False),
        ('w:style/w:qFormat',           True),
        ('w:style/w:qFormat{w:val=0}',  False),
        ('w:style/w:qFormat{w:val=on}', True),
    ])
    def quick_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                     True,  'w:style/w:qFormat'),
        ('w:style/w:qFormat',           False, 'w:style'),
        ('w:style/w:qFormat',           True,  'w:style/w:qFormat'),
        ('w:style/w:qFormat{w:val=0}',  False, 'w:style'),
        ('w:style/w:qFormat{w:val=on}', True,  'w:style/w:qFormat'),
    ])
    def quick_set_fixture(self, request):
        style_cxml, new_value, expected_style_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_style_cxml)
        return style, new_value, expected_xml

    @pytest.fixture(params=[
        ('w:style',                   WD_STYLE_TYPE.PARAGRAPH),
        ('w:style{w:type=paragraph}', WD_STYLE_TYPE.PARAGRAPH),
        ('w:style{w:type=character}', WD_STYLE_TYPE.CHARACTER),
        ('w:style{w:type=numbering}', WD_STYLE_TYPE.LIST),
    ])
    def type_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                           False),
        ('w:style/w:unhideWhenUsed',          True),
        ('w:style/w:unhideWhenUsed{w:val=0}', False),
        ('w:style/w:unhideWhenUsed{w:val=1}', True),
    ])
    def unhide_get_fixture(self, request):
        style_cxml, expected_value = request.param
        style = BaseStyle(element(style_cxml))
        return style, expected_value

    @pytest.fixture(params=[
        ('w:style',                           True,
         'w:style/w:unhideWhenUsed'),
        ('w:style/w:unhideWhenUsed',          False,
         'w:style'),
        ('w:style/w:unhideWhenUsed{w:val=0}', True,
         'w:style/w:unhideWhenUsed'),
        ('w:style/w:unhideWhenUsed{w:val=1}', True,
         'w:style/w:unhideWhenUsed'),
        ('w:style/w:unhideWhenUsed{w:val=1}', False,
         'w:style'),
        ('w:style',                           False,
         'w:style'),
    ])
    def unhide_set_fixture(self, request):
        style_cxml, value, expected_cxml = request.param
        style = BaseStyle(element(style_cxml))
        expected_xml = xml(expected_cxml)
        return style, value, expected_xml


class Describe_CharacterStyle(object):

    def it_knows_which_style_it_is_based_on(self, base_get_fixture):
        style, StyleFactory_, StyleFactory_calls, base_style_ = (
            base_get_fixture
        )
        base_style = style.base_style

        assert StyleFactory_.call_args_list == StyleFactory_calls
        assert base_style == base_style_

    def it_can_change_its_base_style(self, base_set_fixture):
        style, value, expected_xml = base_set_fixture
        style.base_style = value
        assert style._element.xml == expected_xml

    def it_provides_access_to_its_font(self, font_fixture):
        style, Font_, font_ = font_fixture
        font = style.font
        Font_.assert_called_once_with(style._element)
        assert font is font_

    # fixture --------------------------------------------------------

    @pytest.fixture(params=[
        ('w:styles/(w:style{w:styleId=Foo},w:style/w:basedOn{w:val=Foo})',
         1, 0),
        ('w:styles/(w:style{w:styleId=Foo},w:style/w:basedOn{w:val=Bar})',
         1, -1),
        ('w:styles/w:style',
         0, -1),
    ])
    def base_get_fixture(self, request, StyleFactory_):
        styles_cxml, style_idx, base_style_idx = request.param
        styles = element(styles_cxml)
        style = _CharacterStyle(styles[style_idx])
        if base_style_idx >= 0:
            base_style = styles[base_style_idx]
            StyleFactory_calls = [call(base_style)]
            expected_value = StyleFactory_.return_value
        else:
            StyleFactory_calls = []
            expected_value = None
        return style, StyleFactory_, StyleFactory_calls, expected_value

    @pytest.fixture(params=[
        ('w:style',                       'Foo',
         'w:style/w:basedOn{w:val=Foo}'),
        ('w:style/w:basedOn{w:val=Foo}',  'Bar',
         'w:style/w:basedOn{w:val=Bar}'),
        ('w:style/w:basedOn{w:val=Bar}',  None,
         'w:style'),
    ])
    def base_set_fixture(self, request, style_):
        style_cxml, base_style_id, expected_style_cxml = request.param
        style = _CharacterStyle(element(style_cxml))
        style_.style_id = base_style_id
        base_style = style_ if base_style_id is not None else None
        expected_xml = xml(expected_style_cxml)
        return style, base_style, expected_xml

    @pytest.fixture
    def font_fixture(self, Font_, font_):
        style = _CharacterStyle(element('w:style'))
        return style, Font_, font_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def Font_(self, request, font_):
        return class_mock(
            request, 'docx.styles.style.Font', return_value=font_
        )

    @pytest.fixture
    def font_(self, request):
        return instance_mock(request, Font)

    @pytest.fixture
    def style_(self, request):
        return instance_mock(request, BaseStyle)

    @pytest.fixture
    def StyleFactory_(self, request):
        return function_mock(request, 'docx.styles.style.StyleFactory')


class Describe_ParagraphStyle(object):

    def it_knows_its_next_paragraph_style(self, next_get_fixture):
        style, expected_value = next_get_fixture
        assert style.next_paragraph_style == expected_value

    def it_can_change_its_next_paragraph_style(self, next_set_fixture):
        style, next_style, expected_xml = next_set_fixture
        style.next_paragraph_style = next_style
        assert style.element.xml == expected_xml

    def it_provides_access_to_its_paragraph_format(self, parfmt_fixture):
        style, ParagraphFormat_, paragraph_format_ = parfmt_fixture
        paragraph_format = style.paragraph_format
        ParagraphFormat_.assert_called_once_with(style._element)
        assert paragraph_format is paragraph_format_

    # fixtures -------------------------------------------------------

    @pytest.fixture(params=[
        ('H1',   'Body'),
        ('H2',   'H2'),
        ('Body', 'Body'),
        ('Foo',  'Foo'),
    ])
    def next_get_fixture(self, request):
        style_name, next_style_name = request.param
        styles = element(
            'w:styles/('
            'w:style{w:type=paragraph,w:styleId=H1}/w:next{w:val=Body},'
            'w:style{w:type=paragraph,w:styleId=H2}/w:next{w:val=Char},'
            'w:style{w:type=paragraph,w:styleId=Body},'
            'w:style{w:type=paragraph,w:styleId=Foo}/w:next{w:val=Bar},'
            'w:style{w:type=character,w:styleId=Char})'
        )
        style_names = ['H1', 'H2', 'Body', 'Foo', 'Char']
        style_elm = styles[style_names.index(style_name)]
        next_style_elm = styles[style_names.index(next_style_name)]
        style = _ParagraphStyle(style_elm)
        if style_name == 'H1':
            next_style = _ParagraphStyle(next_style_elm)
        else:
            next_style = style
        return style, next_style

    @pytest.fixture(params=[
        ('H', 'B',  'w:style{w:type=paragraph,w:styleId=H}/w:next{w:val=B}'),
        ('H', None, 'w:style{w:type=paragraph,w:styleId=H}'),
        ('H', 'H',  'w:style{w:type=paragraph,w:styleId=H}'),
    ])
    def next_set_fixture(self, request):
        style_name, next_style_name, style_cxml = request.param
        styles = element(
            'w:styles/('
            'w:style{w:type=paragraph,w:styleId=H},'
            'w:style{w:type=paragraph,w:styleId=B})'
        )
        style_elms = {'H': styles[0], 'B': styles[1]}
        style = _ParagraphStyle(style_elms[style_name])
        next_style = (
            None if next_style_name is None else
            _ParagraphStyle(style_elms[next_style_name])
        )
        expected_xml = xml(style_cxml)
        return style, next_style, expected_xml

    @pytest.fixture
    def parfmt_fixture(self, ParagraphFormat_, paragraph_format_):
        style = _ParagraphStyle(element('w:style'))
        return style, ParagraphFormat_, paragraph_format_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def ParagraphFormat_(self, request, paragraph_format_):
        return class_mock(
            request, 'docx.styles.style.ParagraphFormat',
            return_value=paragraph_format_
        )

    @pytest.fixture
    def paragraph_format_(self, request):
        return instance_mock(request, ParagraphFormat)



# encoding: utf-8

"""
Test data builders for text XML elements
"""

from ...unitdata import BaseBuilder
from .shared import CT_StringBuilder


class CT_RowBuilder(BaseBuilder):
    __tag__ = 'w:tr'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:w',)


class CT_TblBuilder(BaseBuilder):
    __tag__ = 'w:tbl'
    __nspfxs__ = ('w',)
    __attrs__ = ()


class CT_TblGridBuilder(BaseBuilder):
    __tag__ = 'w:tblGrid'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:w',)


class CT_TblGridColBuilder(BaseBuilder):
    __tag__ = 'w:gridCol'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:w',)


class CT_TblPrBuilder(BaseBuilder):
    __tag__ = 'w:tblPr'
    __nspfxs__ = ('w',)
    __attrs__ = ()


class CT_TblWidthBuilder(BaseBuilder):
    __tag__ = 'w:tblW'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:w', 'w:type')


class CT_TcBuilder(BaseBuilder):
    __tag__ = 'w:tc'
    __nspfxs__ = ('w',)
    __attrs__ = ('w:id',)


class CT_TcPrBuilder(BaseBuilder):
    __tag__ = 'w:tcPr'
    __nspfxs__ = ('w',)
    __attrs__ = ()


def a_gridCol():
    return CT_TblGridColBuilder()


def a_tbl():
    return CT_TblBuilder()


def a_tblGrid():
    return CT_TblGridBuilder()


def a_tblPr():
    return CT_TblPrBuilder()


def a_tblStyle():
    return CT_StringBuilder('w:tblStyle')


def a_tblW():
    return CT_TblWidthBuilder()


def a_tc():
    return CT_TcBuilder()


def a_tcPr():
    return CT_TcPrBuilder()


def a_tr():
    return CT_RowBuilder()



# encoding: utf-8

"""Unit test suite for the docx.parts.document module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from docx.enum.style import WD_STYLE_TYPE
from docx.opc.constants import RELATIONSHIP_TYPE as RT
from docx.opc.coreprops import CoreProperties
from docx.package import Package
from docx.parts.document import DocumentPart
from docx.parts.hdrftr import FooterPart, HeaderPart
from docx.parts.numbering import NumberingPart
from docx.parts.settings import SettingsPart
from docx.parts.styles import StylesPart
from docx.settings import Settings
from docx.styles.style import BaseStyle
from docx.styles.styles import Styles

from ..oxml.parts.unitdata.document import a_body, a_document
from ..unitutil.mock import class_mock, instance_mock, method_mock, property_mock


class DescribeDocumentPart(object):

    def it_can_add_a_footer_part(self, package_, FooterPart_, footer_part_, relate_to_):
        FooterPart_.new.return_value = footer_part_
        relate_to_.return_value = "rId12"
        document_part = DocumentPart(None, None, None, package_)

        footer_part, rId = document_part.add_footer_part()

        FooterPart_.new.assert_called_once_with(package_)
        relate_to_.assert_called_once_with(document_part, footer_part_, RT.FOOTER)
        assert footer_part is footer_part_
        assert rId == "rId12"

    def it_can_add_a_header_part(self, package_, HeaderPart_, header_part_, relate_to_):
        HeaderPart_.new.return_value = header_part_
        relate_to_.return_value = "rId7"
        document_part = DocumentPart(None, None, None, package_)

        header_part, rId = document_part.add_header_part()

        HeaderPart_.new.assert_called_once_with(package_)
        relate_to_.assert_called_once_with(document_part, header_part_, RT.HEADER)
        assert header_part is header_part_
        assert rId == "rId7"

    def it_can_drop_a_specified_header_part(self, drop_rel_):
        document_part = DocumentPart(None, None, None, None)

        document_part.drop_header_part("rId42")

        drop_rel_.assert_called_once_with(document_part, "rId42")

    def it_provides_access_to_a_footer_part_by_rId(
        self, related_parts_prop_, related_parts_, footer_part_
    ):
        related_parts_prop_.return_value = related_parts_
        related_parts_.__getitem__.return_value = footer_part_
        document_part = DocumentPart(None, None, None, None)

        footer_part = document_part.footer_part("rId9")

        related_parts_.__getitem__.assert_called_once_with("rId9")
        assert footer_part is footer_part_

    def it_provides_access_to_a_header_part_by_rId(
        self, related_parts_prop_, related_parts_, header_part_
    ):
        related_parts_prop_.return_value = related_parts_
        related_parts_.__getitem__.return_value = header_part_
        document_part = DocumentPart(None, None, None, None)

        header_part = document_part.header_part("rId11")

        related_parts_.__getitem__.assert_called_once_with("rId11")
        assert header_part is header_part_

    def it_can_save_the_package_to_a_file(self, save_fixture):
        document, file_ = save_fixture
        document.save(file_)
        document._package.save.assert_called_once_with(file_)

    def it_provides_access_to_the_document_settings(self, settings_fixture):
        document_part, settings_ = settings_fixture
        settings = document_part.settings
        assert settings is settings_

    def it_provides_access_to_the_document_styles(self, styles_fixture):
        document_part, styles_ = styles_fixture
        styles = document_part.styles
        assert styles is styles_

    def it_provides_access_to_its_core_properties(self, core_props_fixture):
        document_part, core_properties_ = core_props_fixture
        core_properties = document_part.core_properties
        assert core_properties is core_properties_

    def it_provides_access_to_the_inline_shapes_in_the_document(
            self, inline_shapes_fixture):
        document, InlineShapes_, body_elm = inline_shapes_fixture
        inline_shapes = document.inline_shapes
        InlineShapes_.assert_called_once_with(body_elm, document)
        assert inline_shapes is InlineShapes_.return_value

    def it_provides_access_to_the_numbering_part(
        self, part_related_by_, numbering_part_
    ):
        part_related_by_.return_value = numbering_part_
        document_part = DocumentPart(None, None, None, None)

        numbering_part = document_part.numbering_part

        part_related_by_.assert_called_once_with(document_part, RT.NUMBERING)
        assert numbering_part is numbering_part_

    def and_it_creates_a_numbering_part_if_not_present(
        self, part_related_by_, relate_to_, NumberingPart_, numbering_part_
    ):
        part_related_by_.side_effect = KeyError
        NumberingPart_.new.return_value = numbering_part_
        document_part = DocumentPart(None, None, None, None)

        numbering_part = document_part.numbering_part

        NumberingPart_.new.assert_called_once_with()
        relate_to_.assert_called_once_with(document_part, numbering_part_, RT.NUMBERING)
        assert numbering_part is numbering_part_

    def it_can_get_a_style_by_id(self, styles_prop_, styles_, style_):
        styles_prop_.return_value = styles_
        styles_.get_by_id.return_value = style_
        document_part = DocumentPart(None, None, None, None)

        style = document_part.get_style("BodyText", WD_STYLE_TYPE.PARAGRAPH)

        styles_.get_by_id.assert_called_once_with("BodyText", WD_STYLE_TYPE.PARAGRAPH)
        assert style is style_

    def it_can_get_the_id_of_a_style(self, style_, styles_prop_, styles_):
        styles_prop_.return_value = styles_
        styles_.get_style_id.return_value = "BodyCharacter"
        document_part = DocumentPart(None, None, None, None)

        style_id = document_part.get_style_id(style_, WD_STYLE_TYPE.CHARACTER)

        styles_.get_style_id.assert_called_once_with(style_, WD_STYLE_TYPE.CHARACTER)
        assert style_id == "BodyCharacter"

    def it_provides_access_to_its_settings_part_to_help(
        self, part_related_by_, settings_part_
    ):
        part_related_by_.return_value = settings_part_
        document_part = DocumentPart(None, None, None, None)

        settings_part = document_part._settings_part

        part_related_by_.assert_called_once_with(document_part, RT.SETTINGS)
        assert settings_part is settings_part_

    def and_it_creates_a_default_settings_part_if_not_present(
        self, package_, part_related_by_, SettingsPart_, settings_part_, relate_to_
    ):
        part_related_by_.side_effect = KeyError
        SettingsPart_.default.return_value = settings_part_
        document_part = DocumentPart(None, None, None, package_)

        settings_part = document_part._settings_part

        SettingsPart_.default.assert_called_once_with(package_)
        relate_to_.assert_called_once_with(document_part, settings_part_, RT.SETTINGS)
        assert settings_part is settings_part_

    def it_provides_access_to_its_styles_part_to_help(
        self, part_related_by_, styles_part_
    ):
        part_related_by_.return_value = styles_part_
        document_part = DocumentPart(None, None, None, None)

        styles_part = document_part._styles_part

        part_related_by_.assert_called_once_with(document_part, RT.STYLES)
        assert styles_part is styles_part_

    def and_it_creates_a_default_styles_part_if_not_present(
        self, package_, part_related_by_, StylesPart_, styles_part_, relate_to_
    ):
        part_related_by_.side_effect = KeyError
        StylesPart_.default.return_value = styles_part_
        document_part = DocumentPart(None, None, None, package_)

        styles_part = document_part._styles_part

        StylesPart_.default.assert_called_once_with(package_)
        relate_to_.assert_called_once_with(document_part, styles_part_, RT.STYLES)
        assert styles_part is styles_part_

    # fixtures -------------------------------------------------------

    @pytest.fixture
    def core_props_fixture(self, package_, core_properties_):
        document_part = DocumentPart(None, None, None, package_)
        package_.core_properties = core_properties_
        return document_part, core_properties_

    @pytest.fixture
    def inline_shapes_fixture(self, request, InlineShapes_):
        document_elm = (
            a_document().with_nsdecls().with_child(
                a_body())
        ).element
        body_elm = document_elm[0]
        document = DocumentPart(None, None, document_elm, None)
        return document, InlineShapes_, body_elm

    @pytest.fixture
    def save_fixture(self, package_):
        document_part = DocumentPart(None, None, None, package_)
        file_ = 'foobar.docx'
        return document_part, file_

    @pytest.fixture
    def settings_fixture(self, _settings_part_prop_, settings_part_,
                         settings_):
        document_part = DocumentPart(None, None, None, None)
        _settings_part_prop_.return_value = settings_part_
        settings_part_.settings = settings_
        return document_part, settings_

    @pytest.fixture
    def styles_fixture(self, _styles_part_prop_, styles_part_, styles_):
        document_part = DocumentPart(None, None, None, None)
        _styles_part_prop_.return_value = styles_part_
        styles_part_.styles = styles_
        return document_part, styles_

    # fixture components ---------------------------------------------

    @pytest.fixture
    def core_properties_(self, request):
        return instance_mock(request, CoreProperties)

    @pytest.fixture
    def drop_rel_(self, request):
        return method_mock(request, DocumentPart, "drop_rel", autospec=True)

    @pytest.fixture
    def FooterPart_(self, request):
        return class_mock(request, 'docx.parts.document.FooterPart')

    @pytest.fixture
    def footer_part_(self, request):
        return instance_mock(request, FooterPart)

    @pytest.fixture
    def HeaderPart_(self, request):
        return class_mock(request, 'docx.parts.document.HeaderPart')

    @pytest.fixture
    def header_part_(self, request):
        return instance_mock(request, HeaderPart)

    @pytest.fixture
    def InlineShapes_(self, request):
        return class_mock(request, 'docx.parts.document.InlineShapes')

    @pytest.fixture
    def NumberingPart_(self, request):
        return class_mock(request, 'docx.parts.document.NumberingPart')

    @pytest.fixture
    def numbering_part_(self, request):
        return instance_mock(request, NumberingPart)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def part_related_by_(self, request):
        return method_mock(request, DocumentPart, 'part_related_by')

    @pytest.fixture
    def relate_to_(self, request):
        return method_mock(request, DocumentPart, 'relate_to')

    @pytest.fixture
    def related_parts_(self, request):
        return instance_mock(request, dict)

    @pytest.fixture
    def related_parts_prop_(self, request):
        return property_mock(request, DocumentPart, 'related_parts')

    @pytest.fixture
    def SettingsPart_(self, request):
        return class_mock(request, 'docx.parts.document.SettingsPart')

    @pytest.fixture
    def settings_(self, request):
        return instance_mock(request, Settings)

    @pytest.fixture
    def settings_part_(self, request):
        return instance_mock(request, SettingsPart)

    @pytest.fixture
    def _settings_part_prop_(self, request):
        return property_mock(request, DocumentPart, '_settings_part')

    @pytest.fixture
    def style_(self, request):
        return instance_mock(request, BaseStyle)

    @pytest.fixture
    def styles_(self, request):
        return instance_mock(request, Styles)

    @pytest.fixture
    def StylesPart_(self, request):
        return class_mock(request, 'docx.parts.document.StylesPart')

    @pytest.fixture
    def styles_part_(self, request):
        return instance_mock(request, StylesPart)

    @pytest.fixture
    def styles_prop_(self, request):
        return property_mock(request, DocumentPart, 'styles')

    @pytest.fixture
    def _styles_part_prop_(self, request):
        return property_mock(request, DocumentPart, '_styles_part')



"""
Custom element classes related to the comments part
"""

from . import OxmlElement
from .simpletypes import ST_DecimalNumber, ST_String
from ..opc.constants import NAMESPACE
from ..text.paragraph import Paragraph
from ..text.run import Run
from .xmlchemy import (
	BaseOxmlElement, OneAndOnlyOne, RequiredAttribute, ZeroOrMore, ZeroOrOne
)

class CT_Com(BaseOxmlElement):
	"""
	A ``<w:comment>`` element, a container for Comment properties 
	"""
	initials = RequiredAttribute('w:initials', ST_String)
	_id = RequiredAttribute('w:id', ST_DecimalNumber)
	date = RequiredAttribute('w:date', ST_String)
	author = RequiredAttribute('w:author', ST_String)
	
	p = ZeroOrOne('w:p', successors=('w:comment',))

	@classmethod
	def new(cls, initials, comm_id, date, author):
		"""
		Return a new ``<w:comment>`` element having _id of *comm_id* and having
		the passed params as meta data 
		"""
		comment = OxmlElement('w:comment')
		comment.initials = initials
		comment.date = date
		comment._id = comm_id
		comment.author = author
		return comment
		
	def _add_p(self, text):
		_p = OxmlElement('w:p')
		_r = _p.add_r()
		run = Run(_r,self)
		run.text = text
		self._insert_p(_p)
		return _p
	
	@property
	def meta(self):
		return [self.author, self.initials, self.date]
	
	@property
	def paragraph(self):
		return Paragraph(self.p, self)
		
	


class CT_Comments(BaseOxmlElement):
	"""
	A ``<w:comments>`` element, a container for Comments properties
	"""
	comment = ZeroOrMore ('w:comment', successors=('w:comments',))

	def add_comment(self,author, initials, date):
		_next_id = self._next_commentId
		comment = CT_Com.new(initials, _next_id, date, author)
		comment = self._insert_comment(comment)

		return comment
	
	@property
	def _next_commentId(self):
		ids = self.xpath('./w:comment/@w:id')
		len(ids)
		_ids = [int(_str) for _str in ids]
		_ids.sort()

		try:
			return _ids[-1] + 2
		except:
			return 0
		
	def get_comment_by_id(self, _id):
		namesapce = NAMESPACE().WML_MAIN
		for c in self.findall('.//w:comment',{'w':namesapce}):
			if c._id == _id:
				return c
		return None
	

class CT_CRS(BaseOxmlElement):
	"""
	A ``<w:commentRangeStart>`` element
	"""
	_id = RequiredAttribute('w:id', ST_DecimalNumber)

	@classmethod
	def new(cls, _id):
		commentRangeStart = OxmlElement('w:commentRangeStart')
		commentRangeStart._id =_id

		return commentRangeStart

class CT_CRE(BaseOxmlElement):
	"""
	A ``w:commentRangeEnd`` element
	"""
	_id = RequiredAttribute('w:id', ST_DecimalNumber)


	@classmethod
	def new(cls, _id):
		commentRangeEnd = OxmlElement('w:commentRangeEnd')
		commentRangeEnd._id =_id
		return commentRangeEnd
	
	
class CT_CRef(BaseOxmlElement):
	"""
	w:commentReference 
	"""
	_id = RequiredAttribute('w:id', ST_DecimalNumber)

	@classmethod
	def new (cls, _id):
		commentReference = OxmlElement('w:commentReference')
		commentReference._id =_id
		return commentReference

	



# encoding: utf-8

"""
Step implementations for styles-related features
"""

from behave import given, then, when

from docx import Document
from docx.enum.style import WD_STYLE_TYPE
from docx.styles.latent import _LatentStyle, LatentStyles
from docx.styles.style import BaseStyle
from docx.text.font import Font
from docx.text.parfmt import ParagraphFormat

from helpers import bool_vals, test_docx, tri_state_vals

style_types = {
    'WD_STYLE_TYPE.CHARACTER': WD_STYLE_TYPE.CHARACTER,
    'WD_STYLE_TYPE.PARAGRAPH': WD_STYLE_TYPE.PARAGRAPH,
    'WD_STYLE_TYPE.LIST':      WD_STYLE_TYPE.LIST,
    'WD_STYLE_TYPE.TABLE':     WD_STYLE_TYPE.TABLE,
}


# given ===================================================

@given('a document having a styles part')
def given_a_document_having_a_styles_part(context):
    docx_path = test_docx('sty-having-styles-part')
    context.document = Document(docx_path)


@given('a document having known styles')
def given_a_document_having_known_styles(context):
    docx_path = test_docx('sty-known-styles')
    document = Document(docx_path)
    context.document = document
    context.style_count = len(document.styles)


@given('a document having no styles part')
def given_a_document_having_no_styles_part(context):
    docx_path = test_docx('sty-having-no-styles-part')
    context.document = Document(docx_path)


@given('a latent style collection')
def given_a_latent_style_collection(context):
    document = Document(test_docx('sty-known-styles'))
    context.latent_styles = document.styles.latent_styles


@given('a latent style having a known name')
def given_a_latent_style_having_a_known_name(context):
    document = Document(test_docx('sty-known-styles'))
    latent_styles = list(document.styles.latent_styles)
    context.latent_style = latent_styles[0]  # should be 'Normal'


@given('a latent style having priority of {setting}')
def given_a_latent_style_having_priority_of_setting(context, setting):
    latent_style_name = {
        '42':         'Normal',
        'no setting': 'Subtitle',
    }[setting]
    document = Document(test_docx('sty-known-styles'))
    latent_styles = document.styles.latent_styles
    context.latent_style = latent_styles[latent_style_name]


@given('a latent style having {prop_name} set {setting}')
def given_a_latent_style_having_prop_setting(context, prop_name, setting):
    latent_style_name = {
        'on':         'Normal',
        'off':        'Title',
        'no setting': 'Subtitle',
    }[setting]
    document = Document(test_docx('sty-known-styles'))
    latent_styles = document.styles.latent_styles
    context.latent_style = latent_styles[latent_style_name]


@given('a latent styles object with known defaults')
def given_a_latent_styles_object_with_known_defaults(context):
    document = Document(test_docx('sty-known-styles'))
    context.latent_styles = document.styles.latent_styles


@given('a style based on {base_style}')
def given_a_style_based_on_setting(context, base_style):
    style_name = {
        'no style': 'Base',
        'Normal':   'Sub Normal',
        'Base':     'Citation',
    }[base_style]
    document = Document(test_docx('sty-known-styles'))
    context.styles = document.styles
    context.style = document.styles[style_name]


@given('a style having a known {attr_name}')
def given_a_style_having_a_known_attr_name(context, attr_name):
    docx_path = test_docx('sty-having-styles-part')
    document = Document(docx_path)
    context.style = document.styles['Normal']


@given('a style having hidden set {setting}')
def given_a_style_having_hidden_set_setting(context, setting):
    document = Document(test_docx('sty-behav-props'))
    style_name = {
        'on':         'Foo',
        'off':        'Bar',
        'no setting': 'Baz',
    }[setting]
    context.style = document.styles[style_name]


@given('a style having locked set {setting}')
def given_a_style_having_locked_setting(context, setting):
    document = Document(test_docx('sty-behav-props'))
    style_name = {
        'on':         'Foo',
        'off':        'Bar',
        'no setting': 'Baz',
    }[setting]
    context.style = document.styles[style_name]


@given('a style having next paragraph style set to {setting}')
def given_a_style_having_next_paragraph_style_setting(context, setting):
    document = Document(test_docx('sty-known-styles'))
    style_name = {
        'Sub Normal': 'Citation',
        'Foobar':     'Sub Normal',
        'Base':       'Foo',
        'no setting': 'Base',
    }[setting]
    context.styles = document.styles
    context.style = document.styles[style_name]


@given('a style having priority of {setting}')
def given_a_style_having_priority_of_setting(context, setting):
    document = Document(test_docx('sty-behav-props'))
    style_name = {
        'no setting': 'Baz',
        '42':         'Foo',
    }[setting]
    context.style = document.styles[style_name]


@given('a style having quick-style set {setting}')
def given_a_style_having_quick_style_setting(context, setting):
    document = Document(test_docx('sty-behav-props'))
    style_name = {
        'on':         'Foo',
        'off':        'Bar',
        'no setting': 'Baz',
    }[setting]
    context.style = document.styles[style_name]


@given('a style having unhide-when-used set {setting}')
def given_a_style_having_unhide_when_used_setting(context, setting):
    document = Document(test_docx('sty-behav-props'))
    style_name = {
        'on':         'Foo',
        'off':        'Bar',
        'no setting': 'Baz',
    }[setting]
    context.style = document.styles[style_name]


@given('a style of type {style_type}')
def given_a_style_of_type(context, style_type):
    document = Document(test_docx('sty-known-styles'))
    name = {
        'WD_STYLE_TYPE.CHARACTER': 'Default Paragraph Font',
        'WD_STYLE_TYPE.LIST':      'No List',
        'WD_STYLE_TYPE.PARAGRAPH': 'Normal',
        'WD_STYLE_TYPE.TABLE':     'Normal Table',
    }[style_type]
    context.style = document.styles[name]


@given('the style collection of a document')
def given_the_style_collection_of_a_document(context):
    document = Document(test_docx('sty-known-styles'))
    context.styles = document.styles


# when =====================================================

@when('I add a latent style named \'Foobar\'')
def when_I_add_a_latent_style_named_Foobar(context):
    latent_styles = context.document.styles.latent_styles
    context.latent_styles = latent_styles
    context.latent_style_count = len(latent_styles)
    latent_styles.add_latent_style('Foobar')


@when('I assign a new name to the style')
def when_I_assign_a_new_name_to_the_style(context):
    context.style.name = 'Foobar'


@when('I assign a new value to style.style_id')
def when_I_assign_a_new_value_to_style_style_id(context):
    context.style.style_id = 'Foo42'


@when('I assign {value} to latent_style.{prop_name}')
def when_I_assign_value_to_latent_style_prop(context, value, prop_name):
    latent_style = context.latent_style
    new_value = (
        tri_state_vals[value] if value in tri_state_vals else int(value)
    )
    setattr(latent_style, prop_name, new_value)


@when('I assign {value} to latent_styles.{prop_name}')
def when_I_assign_value_to_latent_styles_prop(context, value, prop_name):
    latent_styles = context.latent_styles
    new_value = bool_vals[value] if value in bool_vals else int(value)
    setattr(latent_styles, prop_name, new_value)


@when('I assign {value_key} to style.base_style')
def when_I_assign_value_to_style_base_style(context, value_key):
    value = {
        'None':               None,
        'styles[\'Normal\']': context.styles['Normal'],
        'styles[\'Base\']':   context.styles['Base'],
    }[value_key]
    context.style.base_style = value


@when('I assign {value} to style.hidden')
def when_I_assign_value_to_style_hidden(context, value):
    style, new_value = context.style, tri_state_vals[value]
    style.hidden = new_value


@when('I assign {value} to style.locked')
def when_I_assign_value_to_style_locked(context, value):
    style, new_value = context.style, bool_vals[value]
    style.locked = new_value


@when('I assign {value} to style.next_paragraph_style')
def when_I_assign_value_to_style_next_paragraph_style(context, value):
    styles, style = context.styles, context.style
    new_value = None if value == 'None' else styles[value]
    style.next_paragraph_style = new_value


@when('I assign {value} to style.priority')
def when_I_assign_value_to_style_priority(context, value):
    style = context.style
    new_value = None if value == 'None' else int(value)
    style.priority = new_value


@when('I assign {value} to style.quick_style')
def when_I_assign_value_to_style_quick_style(context, value):
    style, new_value = context.style, bool_vals[value]
    style.quick_style = new_value


@when('I assign {value} to style.unhide_when_used')
def when_I_assign_value_to_style_unhide_when_used(context, value):
    style, new_value = context.style, bool_vals[value]
    style.unhide_when_used = new_value


@when('I call add_style(\'{name}\', {type_str}, builtin={builtin_str})')
def when_I_call_add_style(context, name, type_str, builtin_str):
    styles = context.document.styles
    type = style_types[type_str]
    builtin = bool_vals[builtin_str]
    styles.add_style(name, type, builtin=builtin)


@when('I delete a latent style')
def when_I_delete_a_latent_style(context):
    latent_styles = context.document.styles.latent_styles
    context.latent_styles = latent_styles
    context.latent_style_count = len(latent_styles)
    latent_styles['Normal'].delete()


@when('I delete a style')
def when_I_delete_a_style(context):
    context.document.styles['No List'].delete()


# then =====================================================

@then('I can access a latent style by name')
def then_I_can_access_a_latent_style_by_name(context):
    latent_styles = context.latent_styles
    latent_style = latent_styles['Colorful Shading']
    assert isinstance(latent_style, _LatentStyle)


@then('I can access a style by its UI name')
def then_I_can_access_a_style_by_its_UI_name(context):
    styles = context.document.styles
    style = styles['Default Paragraph Font']
    assert isinstance(style, BaseStyle)


@then('I can access a style by style id')
def then_I_can_access_a_style_by_style_id(context):
    styles = context.document.styles
    style = styles['DefaultParagraphFont']
    assert isinstance(style, BaseStyle)


@then('I can iterate over its styles')
def then_I_can_iterate_over_its_styles(context):
    styles = [s for s in context.document.styles]
    assert len(styles) > 0
    assert all(isinstance(s, BaseStyle) for s in styles)


@then('I can iterate over the latent styles')
def then_I_can_iterate_over_the_latent_styles(context):
    latent_styles = [ls for ls in context.latent_styles]
    assert len(latent_styles) == 137
    assert all(isinstance(ls, _LatentStyle) for ls in latent_styles)


@then('latent_style.name is the known name')
def then_latent_style_name_is_the_known_name(context):
    latent_style = context.latent_style
    assert latent_style.name == 'Normal'


@then('latent_style.priority is {value}')
def then_latent_style_priority_is_value(context, value):
    latent_style = context.latent_style
    expected_value = None if value == 'None' else int(value)
    assert latent_style.priority == expected_value


@then('latent_style.{prop_name} is {value}')
def then_latent_style_prop_name_is_value(context, prop_name, value):
    latent_style = context.latent_style
    actual_value = getattr(latent_style, prop_name)
    expected_value = tri_state_vals[value]
    assert actual_value == expected_value


@then('latent_styles[\'Foobar\'] is a latent style')
def then_latentStyles_Foobar_is_a_latent_style(context):
    latent_styles = context.latent_styles
    latent_style = latent_styles['Foobar']
    assert isinstance(latent_style, _LatentStyle)


@then('latent_styles.{prop_name} is {value}')
def then_latent_styles_prop_name_is_value(context, prop_name, value):
    latent_styles = context.latent_styles
    expected_value = bool_vals[value] if value in bool_vals else int(value)
    actual_value = getattr(latent_styles, prop_name)
    assert actual_value == expected_value


@then('len(latent_styles) is 137')
def then_len_latent_styles_is_137(context):
    assert len(context.latent_styles) == 137


@then('len(styles) is {style_count_str}')
def then_len_styles_is_style_count(context, style_count_str):
    assert len(context.document.styles) == int(style_count_str)


@then('style.base_style is {value_key}')
def then_style_base_style_is_value(context, value_key):
    expected_value = {
        'None':               None,
        'styles[\'Normal\']': context.styles['Normal'],
        'styles[\'Base\']':   context.styles['Base'],
    }[value_key]
    style = context.style
    assert style.base_style == expected_value


@then('style.builtin is {builtin_str}')
def then_style_builtin_is_builtin(context, builtin_str):
    style = context.style
    builtin = bool_vals[builtin_str]
    assert style.builtin == builtin


@then('style.font is the Font object for the style')
def then_style_font_is_the_Font_object_for_the_style(context):
    style = context.style
    font = style.font
    assert isinstance(font, Font)
    assert font.element is style.element


@then('style.hidden is {value}')
def then_style_hidden_is_value(context, value):
    style, expected_value = context.style, tri_state_vals[value]
    assert style.hidden is expected_value


@then('style.locked is {value}')
def then_style_locked_is_value(context, value):
    style, expected_value = context.style, bool_vals[value]
    assert style.locked is expected_value


@then('style.name is the {which} name')
def then_style_name_is_the_which_name(context, which):
    expected_name = {
        'known': 'Normal',
        'new':   'Foobar',
    }[which]
    style = context.style
    assert style.name == expected_name


@then('style.next_paragraph_style is {value}')
def then_style_next_paragraph_style_is_value(context, value):
    style, styles = context.style, context.styles
    actual_value = style.next_paragraph_style
    expected_value = styles[value]
    assert actual_value == expected_value, 'got %s' % actual_value


@then('style.paragraph_format is the ParagraphFormat object for the style')
def then_style_paragraph_format_is_the_ParagraphFormat_object(context):
    style = context.style
    paragraph_format = style.paragraph_format
    assert isinstance(paragraph_format, ParagraphFormat)
    assert paragraph_format.element is style.element


@then('style.priority is {value}')
def then_style_priority_is_value(context, value):
    style = context.style
    expected_value = None if value == 'None' else int(value)
    assert style.priority == expected_value


@then('style.quick_style is {value}')
def then_style_quick_style_is_value(context, value):
    style, expected_value = context.style, bool_vals[value]
    assert style.quick_style is expected_value


@then('style.style_id is the {which} style id')
def then_style_style_id_is_the_which_style_id(context, which):
    expected_style_id = {
        'known': 'Normal',
        'new':   'Foo42',
    }[which]
    style = context.style
    assert style.style_id == expected_style_id


@then('style.type is the known type')
def then_style_type_is_the_known_type(context):
    style = context.style
    assert style.type == WD_STYLE_TYPE.PARAGRAPH


@then('style.type is {type_str}')
def then_style_type_is_type(context, type_str):
    style = context.style
    style_type = style_types[type_str]
    assert style.type == style_type


@then('style.unhide_when_used is {value}')
def then_style_unhide_when_used_is_value(context, value):
    style, expected_value = context.style, bool_vals[value]
    assert style.unhide_when_used is expected_value


@then('styles.latent_styles is the LatentStyles object for the document')
def then_styles_latent_styles_is_the_LatentStyles_object(context):
    styles = context.styles
    context.latent_styles = latent_styles = styles.latent_styles
    assert isinstance(latent_styles, LatentStyles)
    assert latent_styles.element is styles.element.latentStyles


@then('styles[\'{name}\'] is a style')
def then_styles_name_is_a_style(context, name):
    styles = context.document.styles
    style = context.style = styles[name]
    assert isinstance(style, BaseStyle)


@then('the deleted latent style is not in the latent styles collection')
def then_the_deleted_latent_style_is_not_in_the_collection(context):
    latent_styles = context.latent_styles
    try:
        latent_styles['Normal']
    except KeyError:
        return
    raise AssertionError('Latent style not deleted')


@then('the deleted style is not in the styles collection')
def then_the_deleted_style_is_not_in_the_styles_collection(context):
    document = context.document
    try:
        document.styles['No List']
    except KeyError:
        return
    raise AssertionError('Style not deleted')


@then('the document has one additional latent style')
def then_the_document_has_one_additional_latent_style(context):
    latent_styles = context.document.styles.latent_styles
    latent_style_count = len(latent_styles)
    expected_count = context.latent_style_count + 1
    assert latent_style_count == expected_count


@then('the document has one additional style')
def then_the_document_has_one_additional_style(context):
    document = context.document
    style_count = len(document.styles)
    expected_style_count = context.style_count + 1
    assert style_count == expected_style_count


@then('the document has one fewer latent styles')
def then_the_document_has_one_fewer_latent_styles(context):
    latent_style_count = len(context.latent_styles)
    expected_count = context.latent_style_count - 1
    assert latent_style_count == expected_count


@then('the document has one fewer styles')
def then_the_document_has_one_fewer_styles(context):
    document = context.document
    style_count = len(document.styles)
    expected_style_count = context.style_count - 1
    assert style_count == expected_style_count



# encoding: utf-8

"""Unit test suite for the docx.parts.hdrftr module"""

from __future__ import absolute_import, division, print_function, unicode_literals

import pytest

from docx.opc.constants import CONTENT_TYPE as CT, RELATIONSHIP_TYPE as RT
from docx.opc.part import PartFactory
from docx.package import Package
from docx.parts.hdrftr import FooterPart, HeaderPart

from ..unitutil.cxml import element
from ..unitutil.mock import function_mock, initializer_mock, instance_mock, method_mock


class DescribeFooterPart(object):

    def it_is_used_by_loader_to_construct_footer_part(
        self, package_, FooterPart_load_, footer_part_
    ):
        partname = "footer1.xml"
        content_type = CT.WML_FOOTER
        reltype = RT.FOOTER
        blob = "<w:ftr/>"
        FooterPart_load_.return_value = footer_part_

        part = PartFactory(partname, content_type, reltype, blob, package_)

        FooterPart_load_.assert_called_once_with(partname, content_type, blob, package_)
        assert part is footer_part_

    def it_can_create_a_new_footer_part(
        self, package_, _default_footer_xml_, parse_xml_, _init_
    ):
        ftr = element("w:ftr")
        package_.next_partname.return_value = "/word/footer24.xml"
        _default_footer_xml_.return_value = "<w:ftr>"
        parse_xml_.return_value = ftr

        footer_part = FooterPart.new(package_)

        package_.next_partname.assert_called_once_with("/word/footer%d.xml")
        _default_footer_xml_.assert_called_once_with()
        parse_xml_.assert_called_once_with("<w:ftr>")
        _init_.assert_called_once_with(
            footer_part, "/word/footer24.xml", CT.WML_FOOTER, ftr, package_
        )

    def it_loads_default_footer_XML_from_a_template_to_help(self):
        # ---tests integration with OS---
        xml_bytes = FooterPart._default_footer_xml()

        assert xml_bytes.startswith(
            b"<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n<w:ftr\n"
        )
        assert len(xml_bytes) == 1395

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _default_footer_xml_(self, request):
        return method_mock(request, FooterPart, "_default_footer_xml", autospec=False)

    @pytest.fixture
    def footer_part_(self, request):
        return instance_mock(request, FooterPart)

    @pytest.fixture
    def FooterPart_load_(self, request):
        return method_mock(request, FooterPart, "load", autospec=False)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, FooterPart)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def parse_xml_(self, request):
        return function_mock(request, "docx.parts.hdrftr.parse_xml")


class DescribeHeaderPart(object):

    def it_is_used_by_loader_to_construct_header_part(
        self, package_, HeaderPart_load_, header_part_
    ):
        partname = "header1.xml"
        content_type = CT.WML_HEADER
        reltype = RT.HEADER
        blob = "<w:hdr/>"
        HeaderPart_load_.return_value = header_part_

        part = PartFactory(partname, content_type, reltype, blob, package_)

        HeaderPart_load_.assert_called_once_with(partname, content_type, blob, package_)
        assert part is header_part_

    def it_can_create_a_new_header_part(
        self, package_, _default_header_xml_, parse_xml_, _init_
    ):
        hdr = element("w:hdr")
        package_.next_partname.return_value = "/word/header42.xml"
        _default_header_xml_.return_value = "<w:hdr>"
        parse_xml_.return_value = hdr

        header_part = HeaderPart.new(package_)

        package_.next_partname.assert_called_once_with("/word/header%d.xml")
        _default_header_xml_.assert_called_once_with()
        parse_xml_.assert_called_once_with("<w:hdr>")
        _init_.assert_called_once_with(
            header_part, "/word/header42.xml", CT.WML_HEADER, hdr, package_
        )

    def it_loads_default_header_XML_from_a_template_to_help(self):
        # ---tests integration with OS---
        xml_bytes = HeaderPart._default_header_xml()

        assert xml_bytes.startswith(
            b"<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n<w:hdr\n"
        )
        assert len(xml_bytes) == 1395

    # fixture components ---------------------------------------------

    @pytest.fixture
    def _default_header_xml_(self, request):
        return method_mock(request, HeaderPart, "_default_header_xml", autospec=False)

    @pytest.fixture
    def HeaderPart_load_(self, request):
        return method_mock(request, HeaderPart, "load", autospec=False)

    @pytest.fixture
    def header_part_(self, request):
        return instance_mock(request, HeaderPart)

    @pytest.fixture
    def _init_(self, request):
        return initializer_mock(request, HeaderPart)

    @pytest.fixture
    def package_(self, request):
        return instance_mock(request, Package)

    @pytest.fixture
    def parse_xml_(self, request):
        return function_mock(request, "docx.parts.hdrftr.parse_xml")
